@@ -73,6 +73,9 @@ void
 history_buffer_add (void *buffer, char *string)
 {
     t_history *new_history, *ptr_history;
+
+    if (!string)
+        return;
     
     if ( !((t_gui_buffer *)(buffer))->history
          || ( ((t_gui_buffer *)(buffer))->history
@@ -99,6 +102,8 @@ history_buffer_add (void *buffer, char *string)
 		&& (((t_gui_buffer *)(buffer))->num_history > cfg_history_max_commands))
 	    {
 		ptr_history = ((t_gui_buffer *)buffer)->last_history->prev_history;
+                if (((t_gui_buffer *)buffer)->ptr_history == ((t_gui_buffer *)buffer)->last_history)
+                    ((t_gui_buffer *)buffer)->ptr_history = ptr_history;
 		((t_gui_buffer *)buffer)->last_history->prev_history->next_history = NULL;
 		if (((t_gui_buffer *)buffer)->last_history->text)
 		    free (((t_gui_buffer *)buffer)->last_history->text);
@@ -119,6 +124,9 @@ history_global_add (char *string)
 {
     t_history *new_history, *ptr_history;
 
+    if (!string)
+        return;
+    
     if (!history_global
         || (history_global
             && ascii_strcasecmp (history_global->text, string) != 0))
@@ -144,6 +152,8 @@ history_global_add (char *string)
 		&& (num_history_global > cfg_history_max_commands))
 	    {
 		ptr_history = history_global_last->prev_history;
+                if (history_global_ptr == history_global_last)
+                    history_global_ptr = ptr_history;
 		history_global_last->prev_history->next_history = NULL;
 		if (history_global_last->text)
 		    free (history_global_last->text);
