@@ -36,6 +36,7 @@
 
 #include "weechat.h"
 #include "session.h"
+#include "log.h"
 #include "../irc/irc.h"
 #include "../gui/gui.h"
 
@@ -147,6 +148,7 @@ session_save_nick (FILE *file, t_irc_nick *nick)
     rc = rc && (session_write_str (file, SESSION_NICK_NICK, nick->nick));
     rc = rc && (session_write_int (file, SESSION_NICK_FLAGS, nick->flags));
     rc = rc && (session_write_int (file, SESSION_NICK_COLOR, nick->color));
+    rc = rc && (session_write_str (file, SESSION_NICK_HOST, nick->host));
     rc = rc && (session_write_id  (file, SESSION_NICK_END));
     return rc;
 }
@@ -171,6 +173,9 @@ session_save_channel (FILE *file, t_irc_channel *channel)
     rc = rc && (session_write_str (file, SESSION_CHAN_KEY, channel->key));
     rc = rc && (session_write_int (file, SESSION_CHAN_NICKS_COUNT, channel->nicks_count));
     rc = rc && (session_write_int (file, SESSION_CHAN_CHECKING_AWAY, channel->checking_away));
+    rc = rc && (session_write_str (file, SESSION_CHAN_AWAY_MESSAGE, channel->away_message));
+    rc = rc && (session_write_int (file, SESSION_CHAN_CYCLE, channel->cycle));
+    rc = rc && (session_write_int (file, SESSION_CHAN_CLOSE, channel->close));
     rc = rc && (session_write_id  (file, SESSION_CHAN_END));
     
     if (!rc)
@@ -222,6 +227,7 @@ session_save_servers (FILE *file)
         rc = rc && (session_write_str (file, SESSION_SERV_NICK3, ptr_server->nick3));
         rc = rc && (session_write_str (file, SESSION_SERV_USERNAME, ptr_server->username));
         rc = rc && (session_write_str (file, SESSION_SERV_REALNAME, ptr_server->realname));
+        rc = rc && (session_write_str (file, SESSION_SERV_HOSTNAME, ptr_server->hostname));
         rc = rc && (session_write_str (file, SESSION_SERV_COMMAND, ptr_server->command));
         rc = rc && (session_write_int (file, SESSION_SERV_COMMAND_DELAY, ptr_server->command_delay));
         rc = rc && (session_write_str (file, SESSION_SERV_AUTOJOIN, ptr_server->autojoin));
@@ -248,6 +254,7 @@ session_save_servers (FILE *file)
 #endif
         rc = rc && (session_write_str (file, SESSION_SERV_UNTERMINATED_MESSAGE, ptr_server->unterminated_message));
         rc = rc && (session_write_str (file, SESSION_SERV_NICK, ptr_server->nick));
+        rc = rc && (session_write_str (file, SESSION_SERV_NICK_MODES, ptr_server->nick_modes));
         rc = rc && (session_write_buf (file, SESSION_SERV_RECONNECT_START, &(ptr_server->reconnect_start), sizeof (time_t)));
         rc = rc && (session_write_int (file, SESSION_SERV_RECONNECT_JOIN, ptr_server->reconnect_join));
         rc = rc && (session_write_int (file, SESSION_SERV_IS_AWAY, ptr_server->is_away));
@@ -361,7 +368,9 @@ session_save_line (FILE *file, t_gui_line *line)
     rc = rc && (session_write_int (file, SESSION_LINE_WITH_MESSAGE, line->line_with_message));
     rc = rc && (session_write_int (file, SESSION_LINE_WITH_HIGHLIGHT, line->line_with_highlight));
     rc = rc && (session_write_str (file, SESSION_LINE_DATA, line->data));
-    rc = rc && (session_write_int (file, SESSION_LINE_OFS_AFTER_DATE, line->ofs_after_date));
+    rc = rc && (session_write_int (file, SESSION_LINE_OFS_AFTER_DATE, line->ofs_start_message));
+    rc = rc && (session_write_int (file, SESSION_LINE_OFS_START_MESSAGE, line->ofs_start_message));
+    rc = rc && (session_write_str (file, SESSION_LINE_NICK, line->nick));
     rc = rc && (session_write_id  (file, SESSION_LINE_END));
     return rc;
 }
@@ -385,7 +394,8 @@ session_save_buffers (FILE *file)
         rc = rc && (session_write_id  (file, SESSION_OBJ_BUFFER));
         rc = rc && (session_write_str (file, SESSION_BUFF_SERVER, SERVER(ptr_buffer) ? SERVER(ptr_buffer)->name : NULL));
         rc = rc && (session_write_str (file, SESSION_BUFF_CHANNEL, CHANNEL(ptr_buffer) ? CHANNEL(ptr_buffer)->name : NULL));
-        rc = rc && (session_write_int (file, SESSION_BUFF_DCC, ptr_buffer->dcc));
+        rc = rc && (session_write_int (file, SESSION_BUFF_TYPE, ptr_buffer->type));
+        rc = rc && (session_write_int (file, SESSION_BUFF_ALL_SERVERS, ptr_buffer->all_servers));
         rc = rc && (session_write_id  (file, SESSION_BUFF_END));
         
         if (!rc)
@@ -464,7 +474,7 @@ session_crash (FILE *file, char *message, ...)
     va_end (argptr);
     
     fclose (file);
-    gui_end ();
+    gui_main_end ();
     fprintf (stderr, "%s %s\n",
              WEECHAT_ERROR, buffer);
     fprintf (stderr,
@@ -473,10 +483,12 @@ session_crash (FILE *file, char *message, ...)
              session_last_read_pos,
              session_last_read_length);
     fprintf (stderr,
-             _("Please send ~/.weechat/%s, ~/.weechat/%s and "
+             _("Please send %s/%s, %s/%s and "
                "above messages to WeeChat developers for support.\n"
                "Be careful, private info may be in these files.\n"),
+             weechat_home,
              WEECHAT_LOG_NAME,
+             weechat_home,
              WEECHAT_SESSION_NAME);
     exit (EXIT_FAILURE);
 }
@@ -852,6 +864,9 @@ session_load_server (FILE *file)
             case SESSION_SERV_REALNAME:
                 rc = rc && (session_read_str (file, &(session_current_server->realname)));
                 break;
+            case SESSION_SERV_HOSTNAME:
+                rc = rc && (session_read_str (file, &(session_current_server->hostname)));
+                break;
             case SESSION_SERV_COMMAND:
                 rc = rc && (session_read_str (file, &(session_current_server->command)));
                 break;
@@ -918,6 +933,9 @@ session_load_server (FILE *file)
             case SESSION_SERV_NICK:
                 rc = rc && (session_read_str (file, &(session_current_server->nick)));
                 break;
+            case SESSION_SERV_NICK_MODES:
+                rc = rc && (session_read_str (file, &(session_current_server->nick_modes)));
+                break;
             case SESSION_SERV_RECONNECT_START:
                 rc = rc && (session_read_buf (file, &(session_current_server->reconnect_start), sizeof (time_t)));
                 break;
@@ -1038,6 +1056,15 @@ session_load_channel (FILE *file)
             case SESSION_CHAN_CHECKING_AWAY:
                 rc = rc && (session_read_int (file, &(session_current_channel->checking_away)));
                 break;
+            case SESSION_CHAN_AWAY_MESSAGE:
+                rc = rc && (session_read_str (file, &(session_current_channel->away_message)));
+                break;
+            case SESSION_CHAN_CYCLE:
+                rc = rc && (session_read_int (file, &(session_current_channel->cycle)));
+                break;
+            case SESSION_CHAN_CLOSE:
+                rc = rc && (session_read_int (file, &(session_current_channel->close)));
+                break;
             default:
                 weechat_log_printf (_("session: warning: ignoring value from "
                                       "channel (object id: %d)\n"));
@@ -1105,6 +1132,9 @@ session_load_nick (FILE *file)
             case SESSION_NICK_COLOR:
                 rc = rc && (session_read_int (file, &(nick->color)));
                 break;
+            case SESSION_NICK_HOST:
+                rc = rc && (session_read_str (file, &(nick->host)));
+                break;
             default:
                 weechat_log_printf (_("session: warning: ignoring value from "
                                       "nick (object id: %d)\n"));
@@ -1183,7 +1213,7 @@ session_load_dcc (FILE *file)
                     return 0;
                 if (string && string[0])
                 {
-                    ptr_channel = channel_search (ptr_server, string);
+                    ptr_channel = channel_search_any (ptr_server, string);
                     if (!ptr_channel)
                     {
                         session_crash (file, _("channel not found for DCC"));
@@ -1328,7 +1358,7 @@ session_load_buffer (FILE *file)
 {
     int object_id, rc;
     char *server_name, *channel_name;
-    int dcc;
+    int buffer_type;
     t_irc_server *ptr_server;
     t_irc_channel *ptr_channel;
     
@@ -1348,18 +1378,18 @@ session_load_buffer (FILE *file)
         return 0;
     }
     
-    /* read dcc */
-    if (!session_read_object (file, SESSION_BUFF_DCC, SESSION_TYPE_INT, &dcc, 0))
+    /* read buffer type */
+    if (!session_read_object (file, SESSION_BUFF_TYPE, SESSION_TYPE_INT, &buffer_type, 0))
     {
-        session_crash (file, _("dcc flag not found for buffer"));
+        session_crash (file, _("buffer type not found"));
         return 0;
     }
     
     /* allocate buffer */
-    weechat_log_printf (_("session: loading buffer (server: %s, channel: %s, dcc: %d)\n"),
+    weechat_log_printf (_("session: loading buffer (server: %s, channel: %s, type: %d)\n"),
                         (server_name) ? server_name : "-",
                         (channel_name) ? channel_name : "-",
-                        dcc);
+                        buffer_type);
     ptr_server = NULL;
     ptr_channel = NULL;
     if (server_name)
@@ -1374,7 +1404,7 @@ session_load_buffer (FILE *file)
     
     if (channel_name)
     {
-        ptr_channel = channel_search (ptr_server, channel_name);
+        ptr_channel = channel_search_any_without_buffer (ptr_server, channel_name);
         if (!ptr_channel)
         {
             session_crash (file, _("channel not found for buffer"));
@@ -1382,7 +1412,8 @@ session_load_buffer (FILE *file)
         }
     }
     
-    session_current_buffer = gui_buffer_new (gui_windows, ptr_server, ptr_channel, dcc, 1);
+    session_current_buffer = gui_buffer_new (gui_windows, ptr_server,
+                                             ptr_channel, buffer_type, 1);
     if (!session_current_buffer)
     {
         session_crash (file, _("can't create new buffer"));
@@ -1407,6 +1438,9 @@ session_load_buffer (FILE *file)
         {
             case SESSION_BUFF_END:
                 return 1;
+            case SESSION_BUFF_ALL_SERVERS:
+                rc = rc && (session_read_int (file, &(session_current_buffer->all_servers)));
+                break;
             default:
                 weechat_log_printf (_("session: warning: ignoring value from "
                                       "buffer (object id: %d)\n"));
@@ -1435,7 +1469,7 @@ session_load_line (FILE *file)
     }
     
     /* allocate line */
-    line = gui_line_new (session_current_buffer);
+    line = gui_buffer_line_new (session_current_buffer);
     if (!line)
     {
         session_crash (file, _("can't create new line"));
@@ -1478,6 +1512,12 @@ session_load_line (FILE *file)
             case SESSION_LINE_OFS_AFTER_DATE:
                 rc = rc && (session_read_int (file, &(line->ofs_after_date)));
                 break;
+            case SESSION_LINE_OFS_START_MESSAGE:
+                rc = rc && (session_read_int (file, &(line->ofs_start_message)));
+                break;
+            case SESSION_LINE_NICK:
+                rc = rc && (session_read_str (file, &(line->nick)));
+                break;
             default:
                 weechat_log_printf (_("session: warning: ignoring value from "
                                       "line (object id: %d)\n"));
@@ -1650,8 +1690,8 @@ session_load (char *filename)
             ptr_server->buffer = gui_buffers;
     }
     
-    gui_switch_to_buffer (gui_windows, gui_buffers);
-    gui_redraw_buffer (gui_current_window->buffer);
+    gui_window_switch_to_buffer (gui_windows, gui_buffers);
+    gui_window_redraw_buffer (gui_current_window->buffer);
     
     fclose (file);
     
