@@ -42,7 +42,6 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <stdarg.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <time.h>
@@ -61,11 +60,13 @@
 #endif
 
 #include "weechat.h"
-#include "weeconfig.h"
+#include "backtrace.h"
 #include "command.h"
 #include "fifo.h"
-#include "utf8.h"
+#include "log.h"
 #include "session.h"
+#include "utf8.h"
+#include "weeconfig.h"
 #include "../irc/irc.h"
 #include "../gui/gui.h"
 
@@ -74,25 +75,54 @@
 #endif
 
 
-char *weechat_argv0 = NULL;     /* WeeChat binary file name (argv[0])               */
-char *weechat_session = NULL;   /* WeeChat session file (for /upgrade command)      */
-time_t weechat_start_time;      /* WeeChat start time (used by /uptime command)     */
-int quit_weechat;               /* = 1 if quit request from user... why ? :'(       */
-int sigsegv = 0;                /* SIGSEGV received?                                */
-char *weechat_home = NULL;      /* WeeChat home dir. (example: /home/toto/.weechat) */
-FILE *weechat_log_file = NULL;  /* WeeChat log file (~/.weechat/weechat.log)        */
+char *weechat_argv0 = NULL;   /* WeeChat binary file name (argv[0])         */
+char *weechat_session = NULL; /* WeeChat session file (for /upgrade cmd)    */
+time_t weechat_start_time;    /* WeeChat start time (used by /uptime cmd)   */
+int quit_weechat;             /* = 1 if quit request from user... why ? :'( */
+int sigsegv = 0;              /* SIGSEGV received?                          */
+char *weechat_home = NULL;    /* WeeChat home dir. (default: ~/.weechat)    */
 
-char *local_charset = NULL;     /* local charset, for example: ISO-8859-1, UTF-8    */
+char *local_charset = NULL;   /* local charset, for ex.: ISO-8859-1, UTF-8  */
 
-int server_cmd_line;            /* at least one server on WeeChat command line      */
-int auto_connect;               /* enabled by default, can by disabled on cmd line  */
-int auto_load_plugins;          /* enabled by default, can by disabled on cmd line  */
+int server_cmd_line;          /* at least 1 server on WeeChat command line  */
+int auto_connect;             /* enabled by default (cmd option to disable) */
+int auto_load_plugins;        /* enabled by default (cmd option to disable) */
 
 #ifdef HAVE_GNUTLS
 gnutls_certificate_credentials gnutls_xcred;   /* gnutls client credentials */
 #endif
 
 
+/*
+ * ascii_tolower: locale independant string conversion to lower case
+ */
+
+void
+ascii_tolower (char *string)
+{
+    while (string && string[0])
+    {
+        if ((string[0] >= 'A') && (string[0] <= 'Z'))
+            string[0] += ('a' - 'A');
+        string++;
+    }
+}
+
+/*
+ * ascii_toupper: locale independant string conversion to upper case
+ */
+
+void
+ascii_toupper (char *string)
+{
+    while (string && string[0])
+    {
+        if ((string[0] >= 'a') && (string[0] <= 'z'))
+            string[0] -= ('a' - 'A');
+        string++;
+    }
+}
+
 /*
  * ascii_strcasecmp: locale and case independent string comparison
  */
@@ -165,49 +195,6 @@ ascii_strncasecmp (char *string1, char *string2, int max)
         return (string1[0]) ? 1 : ((string2[0]) ? -1 : 0);
 }
 
-/*
- * weechat_log_printf: displays a message in WeeChat log (~/.weechat/weechat.log)
- */
-
-void
-weechat_log_printf (char *message, ...)
-{
-    static char buffer[4096];
-    char *ptr_buffer;
-    va_list argptr;
-    static time_t seconds;
-    struct tm *date_tmp;
-    
-    if (!weechat_log_file)
-        return;
-    
-    va_start (argptr, message);
-    vsnprintf (buffer, sizeof (buffer) - 1, message, argptr);
-    va_end (argptr);
-    
-    /* keep only valid chars */
-    ptr_buffer = buffer;
-    while (ptr_buffer[0])
-    {
-        if ((ptr_buffer[0] != '\n')
-            && (ptr_buffer[0] != '\r')
-            && ((unsigned char)(ptr_buffer[0]) < 32))
-            ptr_buffer[0] = '.';
-        ptr_buffer++;
-    }
-    
-    seconds = time (NULL);
-    date_tmp = localtime (&seconds);
-    if (date_tmp)
-        fprintf (weechat_log_file, "[%04d-%02d-%02d %02d:%02d:%02d] %s",
-                 date_tmp->tm_year + 1900, date_tmp->tm_mon + 1, date_tmp->tm_mday,
-                 date_tmp->tm_hour, date_tmp->tm_min, date_tmp->tm_sec,
-                 buffer);
-    else
-        fprintf (weechat_log_file, "%s", buffer);
-    fflush (weechat_log_file);
-}
-
 /*
  * weechat_iconv: convert string to another charset
  */
@@ -262,6 +249,62 @@ weechat_iconv (char *from_code, char *to_code, char *string)
     return outbuf;
 }
 
+/*
+ * weechat_strreplace: replace a string by new one in a string
+ *                     note: returned value has to be free() after use
+ */
+
+char *
+weechat_strreplace (char *string, char *search, char *replace)
+{
+    char *pos, *new_string;
+    int length1, length2, length_new, count;
+    
+    if (!string || !search || !replace)
+        return NULL;
+    
+    length1 = strlen (search);
+    length2 = strlen (replace);
+    
+    /* count number of strings to replace */
+    count = 0;
+    pos = string;
+    while (pos && pos[0] && (pos = strstr (pos, search)))
+    {
+        count++;
+        pos += length1;
+    }
+    
+    /* easy: no string to replace! */
+    if (count == 0)
+        return strdup (string);
+    
+    /* compute needed memory for new string */
+    length_new = strlen (string) - (count * length1) + (count * length2) + 1;
+    
+    /* allocate new string */
+    new_string = (char *)malloc (length_new * sizeof (char));
+    if (!new_string)
+        return strdup (string);
+    
+    /* replace all occurences */
+    new_string[0] = '\0';
+    while (string && string[0])
+    {
+        pos = strstr (string, search);
+        if (pos)
+        {
+            strncat (new_string, string, pos - string);
+            strcat (new_string, replace);
+            pos += length1;
+        }
+        else
+            strcat (new_string, string);
+        string = pos;
+    }
+    return new_string;
+}
+
 /*
  * get_timeval_diff: calculates difference between two times (return in milliseconds)
  */
@@ -300,6 +343,7 @@ weechat_display_usage (char *exec_name)
     printf ("\n\n");
     printf (_("  -a, --no-connect        disable auto-connect to servers at startup\n"
               "  -c, --config            display config file options\n"
+              "  -d, --dir <path>        set WeeChat home directory (default: ~/.weechat)\n"
               "  -f, --key-functions     display WeeChat internal functions for keys\n"
               "  -h, --help              this help\n"
               "  -i, --irc-commands      display IRC commands\n"
@@ -320,7 +364,7 @@ weechat_display_config_options ()
 {
     int i, j, k;
     
-    printf (_("WeeChat configuration options (~/.weechat/weechat.rc):\n\n"));
+    printf (_("WeeChat configuration options (<weechat_home>/weechat.rc):\n\n"));
     for (i = 0; i < CONFIG_NUMBER_SECTIONS; i++)
     {
         if (weechat_options[i])
@@ -477,10 +521,10 @@ weechat_display_keys ()
     printf ("\n");
     for (ptr_key = gui_keys; ptr_key; ptr_key = ptr_key->next_key)
     {
-        expanded_name = gui_key_get_expanded_name (ptr_key->key);
+        expanded_name = gui_keyboard_get_expanded_name (ptr_key->key);
         printf ("* %s => %s\n",
                 (expanded_name) ? expanded_name : ptr_key->key,
-                (ptr_key->function) ? gui_key_function_search_by_ptr (ptr_key->function) : ptr_key->command);
+                (ptr_key->function) ? gui_keyboard_function_search_by_ptr (ptr_key->function) : ptr_key->command);
         if (expanded_name)
             free (expanded_name);
     }
@@ -498,6 +542,7 @@ weechat_parse_args (int argc, char *argv[])
 
     weechat_argv0 = strdup (argv[0]);
     weechat_session = NULL;
+    weechat_home = NULL;
     server_cmd_line = 0;
     auto_connect = 1;
     auto_load_plugins = 1;
@@ -513,6 +558,19 @@ weechat_parse_args (int argc, char *argv[])
             weechat_display_config_options ();
             weechat_shutdown (EXIT_SUCCESS, 0);
         }
+        else if ((strcmp (argv[i], "-d") == 0)
+            || (strcmp (argv[i], "--dir") == 0))
+        {
+            if (i + 1 < argc)
+                weechat_home = strdup (argv[++i]);
+            else
+            {
+                fprintf (stderr,
+                         _("%s missing argument for --dir option\n"),
+                         WEECHAT_ERROR);
+                weechat_shutdown (EXIT_FAILURE, 0);
+            }
+        }
         else if ((strcmp (argv[i], "-f") == 0)
             || (strcmp (argv[i], "--key-functions") == 0))
         {
@@ -586,8 +644,9 @@ weechat_parse_args (int argc, char *argv[])
                                  server_tmp.ipv6, server_tmp.ssl,
                                  server_tmp.password, server_tmp.nick1,
                                  server_tmp.nick2, server_tmp.nick3,
-                                 NULL, NULL, NULL, 0, server_tmp.autojoin, 1, NULL,
-                                 NULL, NULL, NULL))
+                                 NULL, NULL, NULL, NULL, 0,
+                                 server_tmp.autojoin, 1, NULL, NULL,
+                                 NULL, NULL))
                     fprintf (stderr, _("%s unable to create server ('%s'), ignored\n"),
                              WEECHAT_WARNING, argv[i]);
                 server_destroy (&server_tmp);
@@ -626,56 +685,86 @@ weechat_create_dir (char *directory)
 }
 
 /*
- * weechat_create_home_dirs: create WeeChat directories (if not found)
+ * weechat_create_home_dirs: create WeeChat directories
  */
 
 void
 weechat_create_home_dirs ()
 {
-    char *ptr_home, *dir_name;
+    char *ptr_home;
     int dir_length;
-    
-    ptr_home = getenv ("HOME");
-    if (!ptr_home)
-    {
-        fprintf (stderr, _("%s unable to get HOME directory\n"),
-                 WEECHAT_ERROR);
-        weechat_shutdown (EXIT_FAILURE, 0);
-    }
-    dir_length = strlen (ptr_home) + 10;
-    weechat_home =
-        (char *) malloc (dir_length * sizeof (char));
+
     if (!weechat_home)
     {
-        fprintf (stderr, _("%s not enough memory for home directory\n"),
-                 WEECHAT_ERROR);
-        weechat_shutdown (EXIT_FAILURE, 0);
+        ptr_home = getenv ("HOME");
+        if (!ptr_home)
+        {
+            fprintf (stderr, _("%s unable to get HOME directory\n"),
+                     WEECHAT_ERROR);
+            weechat_shutdown (EXIT_FAILURE, 0);
+        }
+        dir_length = strlen (ptr_home) + 10;
+        weechat_home =
+            (char *) malloc (dir_length * sizeof (char));
+        if (!weechat_home)
+        {
+            fprintf (stderr, _("%s not enough memory for home directory\n"),
+                     WEECHAT_ERROR);
+            weechat_shutdown (EXIT_FAILURE, 0);
+        }
+        snprintf (weechat_home, dir_length, "%s%s.weechat", ptr_home,
+                  DIR_SEPARATOR);
     }
-    snprintf (weechat_home, dir_length, "%s%s.weechat", ptr_home,
-              DIR_SEPARATOR);
     
-    /* create home directory "~/.weechat" ; error is fatal */
+    /* create home directory; error is fatal */
     if (!weechat_create_dir (weechat_home))
     {
-        fprintf (stderr, _("%s unable to create ~/.weechat directory\n"),
-                 WEECHAT_ERROR);
+        fprintf (stderr, _("%s unable to create \"%s\" directory\n"),
+                 WEECHAT_ERROR, weechat_home);
         weechat_shutdown (EXIT_FAILURE, 0);
     }
+}
+
+/*
+ * weechat_create_config_dirs: create config directories (read from config file)
+ */
+
+void
+weechat_create_config_dirs ()
+{
+    char *dir1, *dir2;
     
-    dir_length = strlen (weechat_home) + 64;
-    dir_name = (char *) malloc (dir_length * sizeof (char));
-    
-    /* create "~/.weechat/logs" */
-    snprintf (dir_name, dir_length, "%s%s%s", weechat_home, DIR_SEPARATOR,
-              "logs");
-    if (!weechat_create_dir (dir_name))
+    /* create logs directory" */
+    dir1 = weechat_strreplace (cfg_log_path, "~", getenv ("HOME"));
+    dir2 = weechat_strreplace (dir1, "%h", weechat_home);
+    if (weechat_create_dir (dir2))
     {
-        fprintf (stderr, _("%s unable to create ~/.weechat/logs directory\n"),
-                 WEECHAT_WARNING);
+        if (strcmp (dir2, getenv ("HOME")) != 0)
+            chmod (dir2, 0700);
     }
-    chmod (dir_name, 0700);
+    else
+        fprintf (stderr, _("%s unable to create \"%s\" directory\n"),
+                 WEECHAT_WARNING, dir2);
+    if (dir1)
+        free (dir1);
+    if (dir2)
+        free (dir2);
     
-    free (dir_name);
+    /* create DCC download directory */
+    dir1 = weechat_strreplace (cfg_dcc_download_path, "~", getenv ("HOME"));
+    dir2 = weechat_strreplace (dir1, "%h", weechat_home);
+    if (weechat_create_dir (dir2))
+    {
+        if (strcmp (dir2, getenv ("HOME")) != 0)
+            chmod (dir2, 0700);
+    }
+    else
+        fprintf (stderr, _("%s unable to create \"%s\" directory\n"),
+                 WEECHAT_WARNING, dir2);
+    if (dir1)
+        free (dir1);
+    if (dir2)
+        free (dir2);
 }
 
 /*
@@ -700,27 +789,6 @@ weechat_init_vars ()
 #endif
 }
 
-/*
- * weechat_init_log: initialize log file
- */
-
-void
-weechat_init_log ()
-{
-    int filename_length;
-    char *filename;
-    
-    filename_length = strlen (weechat_home) + 64;
-    filename =
-        (char *) malloc (filename_length * sizeof (char));
-    snprintf (filename, filename_length, "%s/" WEECHAT_LOG_NAME, weechat_home);
-    if ((weechat_log_file = fopen (filename, "wt")) == NULL)
-        fprintf (stderr,
-                 _("%s unable to create/append to log file (~/.weechat/%s)"),
-                 WEECHAT_WARNING, WEECHAT_LOG_NAME);
-    free (filename);
-}
-
 /*
  * weechat_config_read: read WeeChat config file
  */
@@ -806,8 +874,7 @@ weechat_shutdown (int return_code, int crash)
     fifo_remove ();
     if (weechat_home)
         free (weechat_home);
-    if (weechat_log_file)
-        fclose (weechat_log_file);
+    weechat_log_close ();
     if (local_charset)
         free (local_charset);
     alias_free_all ();
@@ -824,7 +891,7 @@ weechat_shutdown (int return_code, int crash)
 }
 
 /*
- * weechat_dump writes dump to WeeChat log file
+ * weechat_dump: write dump to WeeChat log file
  */
 
 void
@@ -921,7 +988,7 @@ weechat_dump (int crash)
 }
 
 /*
- * weechat_sigsegv: SIGSEGV handler: save crash log to ~/.weechat/weechat.log and exit
+ * weechat_sigsegv: SIGSEGV handler: save crash log to <weechat_home>/weechat.log and exit
  */
 
 void
@@ -930,13 +997,27 @@ weechat_sigsegv ()
     weechat_dump (1);
     dcc_end ();
     server_free_all ();
-    gui_end ();
+    gui_main_end ();
+
     fprintf (stderr, "\n");
-    fprintf (stderr, "*** Very bad! WeeChat has crashed (SIGSEGV received)\n");
-    fprintf (stderr, "*** Full crash dump was saved to ~/.weechat/weechat.log file\n");
-    fprintf (stderr, "*** Please send this file to WeeChat developers.\n");
-    fprintf (stderr, "*** (be careful, private info may be in this file since\n");
-    fprintf (stderr, "*** part of chats are displayed, so remove lines if needed)\n\n");
+    fprintf (stderr, "*** Very bad! WeeChat is crashing (SIGSEGV received)\n");
+    fprintf (stderr, "*** (%s, compiled on %s %s)\n",
+             PACKAGE_STRING, __DATE__, __TIME__);
+    if (!weechat_log_crash_rename ())
+        fprintf (stderr, "*** Full crash dump was saved to %s/weechat.log file.\n",
+                 weechat_home);
+    fprintf (stderr, "***\n");
+    fprintf (stderr, "*** Please help WeeChat developers to fix this bug:\n");
+    fprintf (stderr, "***   1. If you have a core file, please run:  gdb weechat-curses core\n");
+    fprintf (stderr, "***      then issue \"bt\" command and send result to developers\n");
+    fprintf (stderr, "***      To enable core files with bash shell: ulimit -c 10000\n");
+    fprintf (stderr, "***   2. Otherwise send backtrace (below) and weechat.log\n");
+    fprintf (stderr, "***      (be careful, private info may be in this file since\n");
+    fprintf (stderr, "***      part of chats are displayed, so remove lines if needed)\n\n");
+    
+    weechat_backtrace ();
+    
+    /* shutdown with error code */
     weechat_shutdown (EXIT_FAILURE, 1);
 }
 
@@ -961,20 +1042,21 @@ main (int argc, char *argv[])
     signal (SIGQUIT, SIG_IGN);          /* ignore SIGQUIT signal            */
     signal (SIGPIPE, SIG_IGN);          /* ignore SIGPIPE signal            */
     signal (SIGSEGV, weechat_sigsegv);  /* crash dump when SIGSEGV received */
-    gui_pre_init (&argc, &argv);        /* pre-initiliaze interface         */
+    gui_main_pre_init (&argc, &argv);   /* pre-initiliaze interface         */
     weechat_init_vars ();               /* initialize some variables        */
-    gui_key_init ();                    /* init keyb. (default key bindings)*/
+    gui_keyboard_init ();               /* init keyb. (default key bindings)*/
     weechat_parse_args (argc, argv);    /* parse command line args          */
     weechat_create_home_dirs ();        /* create WeeChat directories       */
-    weechat_init_log ();                /* init log file                    */
+    weechat_log_init ();                /* init log file                    */
     command_index_build ();             /* build cmd index for completion   */
     weechat_config_read ();             /* read configuration               */
+    weechat_create_config_dirs ();      /* create config directories        */
     utf8_init ();                       /* init UTF-8 in WeeChat            */
-    gui_init ();                        /* init WeeChat interface           */
-    weechat_welcome_message ();         /* display WeeChat welcome message  */
+    gui_main_init ();                   /* init WeeChat interface           */
     fifo_create ();                     /* FIFO pipe for remote control     */
     if (weechat_session)
         session_load (weechat_session); /* load previous session if asked   */
+    weechat_welcome_message ();         /* display WeeChat welcome message  */
 #ifdef PLUGINS
     plugin_init (auto_load_plugins);    /* init plugin interface(s)         */
 #endif
@@ -992,7 +1074,7 @@ main (int argc, char *argv[])
     command_index_free ();              /* free commands index              */
     dcc_end ();                         /* remove all DCC                   */
     server_free_all ();                 /* free all servers                 */
-    gui_end ();                         /* shut down WeeChat GUI            */
+    gui_main_end ();                    /* shut down WeeChat GUI            */
     weechat_shutdown (EXIT_SUCCESS, 0); /* quit WeeChat (oh no, why?)       */
     
     return EXIT_SUCCESS;                /* make gcc happy (never executed)  */
