@@ -54,6 +54,10 @@
 #define CFG_LOOK_NICKLIST_TOP    2
 #define CFG_LOOK_NICKLIST_BOTTOM 3
 
+#define CFG_LOOK_ALIGN_NICK_NONE  0
+#define CFG_LOOK_ALIGN_NICK_LEFT  1
+#define CFG_LOOK_ALIGN_NICK_RIGHT 2
+
 #define CFG_IRC_DISPLAY_AWAY_OFF     0
 #define CFG_IRC_DISPLAY_AWAY_LOCAL   1
 #define CFG_IRC_DISPLAY_AWAY_CHANNEL 2
@@ -92,6 +96,8 @@ extern char *cfg_look_charset_decode_utf;
 extern char *cfg_look_charset_encode;
 extern char *cfg_look_charset_internal;
 extern int cfg_look_one_server_buffer;
+extern int cfg_look_open_near_server;
+extern int cfg_look_scroll_amount;
 extern char *cfg_look_buffer_timestamp;
 extern int cfg_look_color_nicks_number;
 extern int cfg_look_color_actions;
@@ -102,8 +108,15 @@ extern int cfg_look_nicklist_max_size;
 extern int cfg_look_nickmode;
 extern int cfg_look_nickmode_empty;
 extern char *cfg_look_no_nickname;
-extern char *cfg_look_completor;
+extern char *cfg_look_nick_prefix;
+extern char *cfg_look_nick_suffix;
+extern int cfg_look_align_nick;
+extern int cfg_look_align_other;
+extern int cfg_look_align_size;
+extern int cfg_look_align_size_max;
+extern char *cfg_look_nick_completor;
 extern char *cfg_look_nick_completion_ignore;
+extern int cfg_look_nick_complete_first;
 extern int cfg_look_infobar;
 extern char *cfg_look_infobar_timestamp;
 extern int cfg_look_infobar_seconds;
@@ -114,6 +127,7 @@ extern int cfg_look_hotlist_names_length;
 extern int cfg_look_day_change;
 extern char *cfg_look_day_change_timestamp;
 extern char *cfg_look_read_marker;
+extern char *cfg_look_input_format;
 
 extern int cfg_col_real_white;
 extern int cfg_col_separator;
@@ -149,6 +163,7 @@ extern int cfg_col_infobar_delimiters;
 extern int cfg_col_infobar_highlight;
 extern int cfg_col_infobar_bg;
 extern int cfg_col_input;
+extern int cfg_col_input_server;
 extern int cfg_col_input_channel;
 extern int cfg_col_input_nick;
 extern int cfg_col_input_delimiters;
@@ -187,10 +202,12 @@ extern char *cfg_log_timestamp;
 extern int cfg_log_hide_nickserv_pwd;
 
 extern int cfg_irc_display_away;
+extern int cfg_irc_show_away_once;
 extern char *cfg_irc_default_msg_part;
 extern char *cfg_irc_default_msg_quit;
 extern int cfg_irc_notice_as_pv;
 extern int cfg_irc_away_check;
+extern int cfg_irc_away_check_max_nicks;
 extern int cfg_irc_lag_check;
 extern int cfg_irc_lag_min_show;
 extern int cfg_irc_lag_disconnect;
