@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2005 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /* weelist.c: sorted lists management */
@@ -42,7 +42,7 @@ weelist_search (t_weelist *weelist, char *data)
     
     for (ptr_weelist = weelist; ptr_weelist; ptr_weelist = ptr_weelist->next_weelist)
     {
-        if (strcasecmp (data, ptr_weelist->data) == 0)
+        if (ascii_strcasecmp (data, ptr_weelist->data) == 0)
             return ptr_weelist;
     }
     /* word not found in list */
@@ -60,7 +60,7 @@ weelist_find_pos (t_weelist *weelist, char *data)
     
     for (ptr_weelist = weelist; ptr_weelist; ptr_weelist = ptr_weelist->next_weelist)
     {
-        if (strcasecmp (data, ptr_weelist->data) < 0)
+        if (ascii_strcasecmp (data, ptr_weelist->data) < 0)
             return ptr_weelist;
     }
     /* position not found, best position is at the end */
@@ -140,6 +140,9 @@ weelist_remove (t_weelist **weelist, t_weelist **last_weelist, t_weelist *elemen
 {
     t_weelist *new_weelist;
     
+    if (!element)
+        return;
+    
     /* remove element from list */
     if (*last_weelist == element)
         *last_weelist = element->prev_weelist;
