@@ -76,7 +76,7 @@ gui_action_clipboard_paste (t_gui_window *window)
     {
         gui_insert_string_input (window, gui_input_clipboard, -1);
         window->buffer->input_buffer_pos += utf8_strlen (gui_input_clipboard);
-        gui_draw_buffer_input (window->buffer, 0);
+        gui_input_draw (window->buffer, 0);
         window->buffer->completion.position = -1;
     }
 }
@@ -95,20 +95,24 @@ gui_action_return (t_gui_window *window)
         if (window->buffer->input_buffer_size > 0)
         {
             window->buffer->input_buffer[window->buffer->input_buffer_size] = '\0';
+            window->buffer->input_buffer_color_mask[window->buffer->input_buffer_size] = '\0';
             command = strdup (window->buffer->input_buffer);
             if (!command)
                 return;
             history_buffer_add (window->buffer, window->buffer->input_buffer);
             history_global_add (window->buffer->input_buffer);
             window->buffer->input_buffer[0] = '\0';
+            window->buffer->input_buffer_color_mask[0] = '\0';
             window->buffer->input_buffer_size = 0;
             window->buffer->input_buffer_length = 0;
             window->buffer->input_buffer_pos = 0;
             window->buffer->input_buffer_1st_display = 0;
             window->buffer->completion.position = -1;
             window->buffer->ptr_history = NULL;
-            gui_draw_buffer_input (window->buffer, 0);
-            user_command (window->buffer, SERVER(window->buffer), command);
+            gui_input_optimize_size (window->buffer);
+            gui_input_draw (window->buffer, 0);
+            user_command (SERVER(window->buffer), CHANNEL(window->buffer),
+                          command, 0);
             free (command);
         }
     }
@@ -121,100 +125,32 @@ gui_action_return (t_gui_window *window)
 void
 gui_action_tab (t_gui_window *window)
 {
-    int i;
-
     if (window->buffer->has_input)
     {
-        completion_search (&(window->buffer->completion),
+        completion_search (&(window->buffer->completion), 1,
                            window->buffer->input_buffer,
                            window->buffer->input_buffer_size,
                            utf8_real_pos (window->buffer->input_buffer,
                                           window->buffer->input_buffer_pos));
-        
-        if (window->buffer->completion.word_found)
-        {
-            /* replace word with new completed word into input buffer */
-            if (window->buffer->completion.diff_size > 0)
-            {
-                window->buffer->input_buffer_size +=
-                    window->buffer->completion.diff_size;
-                window->buffer->input_buffer_length +=
-                    window->buffer->completion.diff_length;
-                gui_optimize_input_buffer_size (window->buffer);
-                window->buffer->input_buffer[window->buffer->input_buffer_size] = '\0';
-                for (i = window->buffer->input_buffer_size - 1;
-                     i >=  window->buffer->completion.position_replace +
-                         (int)strlen (window->buffer->completion.word_found); i--)
-                    window->buffer->input_buffer[i] =
-                        window->buffer->input_buffer[i - window->buffer->completion.diff_size];
-            }
-            else
-            {
-                for (i = window->buffer->completion.position_replace +
-                         strlen (window->buffer->completion.word_found);
-                     i < window->buffer->input_buffer_size; i++)
-                    window->buffer->input_buffer[i] =
-                        window->buffer->input_buffer[i - window->buffer->completion.diff_size];
-                window->buffer->input_buffer_size +=
-                    window->buffer->completion.diff_size;
-                window->buffer->input_buffer_length +=
-                    window->buffer->completion.diff_length;
-                gui_optimize_input_buffer_size (window->buffer);
-                window->buffer->input_buffer[window->buffer->input_buffer_size] = '\0';
-            }
-            
-            strncpy (window->buffer->input_buffer + window->buffer->completion.position_replace,
-                     window->buffer->completion.word_found,
-                     strlen (window->buffer->completion.word_found));
-            window->buffer->input_buffer_pos =
-                utf8_pos (window->buffer->input_buffer,
-                          window->buffer->completion.position_replace) +
-                utf8_strlen (window->buffer->completion.word_found);
-                        
-            /* position is < 0 this means only one word was found to complete,
-               so reinit to stop completion */
-            if (window->buffer->completion.position >= 0)
-                window->buffer->completion.position =
-                    utf8_real_pos (window->buffer->input_buffer,
-                                   window->buffer->input_buffer_pos);
-                        
-            /* add space or completor to the end of completion, if needed */
-            if ((window->buffer->completion.context == COMPLETION_COMMAND)
-                || (window->buffer->completion.context == COMPLETION_COMMAND_ARG))
-            {
-                if (window->buffer->input_buffer[utf8_real_pos (window->buffer->input_buffer,
-                                                                window->buffer->input_buffer_pos)] != ' ')
-                    gui_insert_string_input (window, " ",
-                                             window->buffer->input_buffer_pos);
-                if (window->buffer->completion.position >= 0)
-                    window->buffer->completion.position++;
-                window->buffer->input_buffer_pos++;
-            }
-            else
-            {
-                /* add nick completor if position 0 and completing nick */
-                if ((window->buffer->completion.base_word_pos == 0)
-                    && (window->buffer->completion.context == COMPLETION_NICK))
-                {
-                    if (strncmp (utf8_add_offset (window->buffer->input_buffer,
-                                                  window->buffer->input_buffer_pos),
-                                 cfg_look_completor, strlen (cfg_look_completor)) != 0)
-                        gui_insert_string_input (window, cfg_look_completor,
-                                                 window->buffer->input_buffer_pos);
-                    if (window->buffer->completion.position >= 0)
-                        window->buffer->completion.position += strlen (cfg_look_completor);
-                    window->buffer->input_buffer_pos += utf8_strlen (cfg_look_completor);
-                    if (window->buffer->input_buffer[utf8_real_pos (window->buffer->input_buffer,
-                                                                    window->buffer->input_buffer_pos)] != ' ')
-                        gui_insert_string_input (window, " ",
-                                                 window->buffer->input_buffer_pos);
-                    if (window->buffer->completion.position >= 0)
-                        window->buffer->completion.position++;
-                    window->buffer->input_buffer_pos++;
-                }
-            }
-            gui_draw_buffer_input (window->buffer, 0);
-        }
+        gui_input_complete (window);
+    }
+}
+
+/*
+ * gui_action_tab_previous: shift-tab key => find previous completion
+ */
+
+void
+gui_action_tab_previous (t_gui_window *window)
+{
+    if (window->buffer->has_input)
+    {
+        completion_search (&(window->buffer->completion), -1,
+                           window->buffer->input_buffer,
+                           window->buffer->input_buffer_size,
+                           utf8_real_pos (window->buffer->input_buffer,
+                                          window->buffer->input_buffer_pos));
+        gui_input_complete (window);
     }
 }
 
@@ -237,13 +173,14 @@ gui_action_backspace (t_gui_window *window)
             pos_last = utf8_prev_char (window->buffer->input_buffer, pos);
             char_size = pos - pos_last;
             size_to_move = strlen (pos);
-            memmove (pos_last, pos, size_to_move);
+            gui_input_move (window->buffer, pos_last, pos, size_to_move);
             window->buffer->input_buffer_size -= char_size;
             window->buffer->input_buffer_length--;
             window->buffer->input_buffer_pos--;
             window->buffer->input_buffer[window->buffer->input_buffer_size] = '\0';
-            gui_optimize_input_buffer_size (window->buffer);
-            gui_draw_buffer_input (window->buffer, 0);
+            window->buffer->input_buffer_color_mask[window->buffer->input_buffer_size] = '\0';
+            gui_input_optimize_size (window->buffer);
+            gui_input_draw (window->buffer, 0);
             window->buffer->completion.position = -1;
         }
     }
@@ -269,12 +206,13 @@ gui_action_delete (t_gui_window *window)
             pos_next = utf8_next_char (pos);
             char_size = pos_next - pos;
             size_to_move = strlen (pos_next);
-            memmove (pos, pos_next, size_to_move);
+            gui_input_move (window->buffer, pos, pos_next, size_to_move);
             window->buffer->input_buffer_size -= char_size;
             window->buffer->input_buffer_length--;
             window->buffer->input_buffer[window->buffer->input_buffer_size] = '\0';
-            gui_optimize_input_buffer_size (window->buffer);
-            gui_draw_buffer_input (window->buffer, 0);
+            window->buffer->input_buffer_color_mask[window->buffer->input_buffer_size] = '\0';
+            gui_input_optimize_size (window->buffer);
+            gui_input_draw (window->buffer, 0);
             window->buffer->completion.position = -1;
         }
     }
@@ -326,14 +264,15 @@ gui_action_delete_previous_word (t_gui_window *window)
             
             gui_action_clipboard_copy (string, size_deleted);
             
-            memmove (string, string + size_deleted, strlen (string + size_deleted));
+            gui_input_move (window->buffer, string, string + size_deleted, strlen (string + size_deleted));
             
             window->buffer->input_buffer_size -= size_deleted;
             window->buffer->input_buffer_length -= length_deleted;
             window->buffer->input_buffer[window->buffer->input_buffer_size] = '\0';
+            window->buffer->input_buffer_color_mask[window->buffer->input_buffer_size] = '\0';
             window->buffer->input_buffer_pos -= length_deleted;
-            gui_optimize_input_buffer_size (window->buffer);
-            gui_draw_buffer_input (window->buffer, 0);
+            gui_input_optimize_size (window->buffer);
+            gui_input_draw (window->buffer, 0);
             window->buffer->completion.position = -1;
         }
     }
@@ -366,13 +305,14 @@ gui_action_delete_next_word (t_gui_window *window)
         
         gui_action_clipboard_copy(start, size_deleted);
         
-        memmove (start, string, strlen (string));
+        gui_input_move (window->buffer, start, string, strlen (string));
         
         window->buffer->input_buffer_size -= size_deleted;
         window->buffer->input_buffer_length -= length_deleted;
         window->buffer->input_buffer[window->buffer->input_buffer_size] = '\0';
-        gui_optimize_input_buffer_size (window->buffer);
-        gui_draw_buffer_input (window->buffer, 0);
+        window->buffer->input_buffer_color_mask[window->buffer->input_buffer_size] = '\0';
+        gui_input_optimize_size (window->buffer);
+        gui_input_draw (window->buffer, 0);
         window->buffer->completion.position = -1;
     }
 }
@@ -384,7 +324,7 @@ gui_action_delete_next_word (t_gui_window *window)
 void
 gui_action_delete_begin_of_line (t_gui_window *window)
 {
-    int length_deleted, size_deleted;
+    int length_deleted, size_deleted, pos_start;
     char *start;
     
     if (window->buffer->has_input)
@@ -393,19 +333,21 @@ gui_action_delete_begin_of_line (t_gui_window *window)
         {
             start = utf8_add_offset (window->buffer->input_buffer,
                                      window->buffer->input_buffer_pos);
+            pos_start = start - window->buffer->input_buffer;
             size_deleted = start - window->buffer->input_buffer;
             length_deleted = utf8_strnlen (window->buffer->input_buffer, size_deleted);
             gui_action_clipboard_copy (window->buffer->input_buffer,
                                        start - window->buffer->input_buffer);
             
-            memmove (window->buffer->input_buffer, start, strlen (start));
+            gui_input_move (window->buffer, window->buffer->input_buffer, start, strlen (start));
             
             window->buffer->input_buffer_size -= size_deleted;
             window->buffer->input_buffer_length -= length_deleted;
             window->buffer->input_buffer[window->buffer->input_buffer_size] = '\0';
+            window->buffer->input_buffer_color_mask[window->buffer->input_buffer_size] = '\0';
             window->buffer->input_buffer_pos = 0;
-            gui_optimize_input_buffer_size (window->buffer);
-            gui_draw_buffer_input (window->buffer, 0);
+            gui_input_optimize_size (window->buffer);
+            gui_input_draw (window->buffer, 0);
             window->buffer->completion.position = -1;
         }
     }
@@ -419,20 +361,22 @@ void
 gui_action_delete_end_of_line (t_gui_window *window)
 {
     char *start;
-    int size_deleted, length_deleted;
+    int size_deleted, length_deleted, pos_start;
     
     if (window->buffer->has_input)
     {
         start = utf8_add_offset (window->buffer->input_buffer,
                                  window->buffer->input_buffer_pos);
+        pos_start = start - window->buffer->input_buffer;
         size_deleted = strlen (start);
         length_deleted = utf8_strlen (start);
         gui_action_clipboard_copy (start, size_deleted);
         start[0] = '\0';
+        window->buffer->input_buffer_color_mask[pos_start] = '\0';
         window->buffer->input_buffer_size = strlen (window->buffer->input_buffer);
         window->buffer->input_buffer_length = utf8_strlen (window->buffer->input_buffer);
-        gui_optimize_input_buffer_size (window->buffer);
-        gui_draw_buffer_input (window->buffer, 0);
+        gui_input_optimize_size (window->buffer);
+        gui_input_draw (window->buffer, 0);
         window->buffer->completion.position = -1;
     }
 }
@@ -447,11 +391,12 @@ gui_action_delete_line (t_gui_window *window)
     if (window->buffer->has_input)
     {
         window->buffer->input_buffer[0] = '\0';
+        window->buffer->input_buffer_color_mask[0] = '\0';
         window->buffer->input_buffer_size = 0;
         window->buffer->input_buffer_length = 0;
         window->buffer->input_buffer_pos = 0;
-        gui_optimize_input_buffer_size (window->buffer);
-        gui_draw_buffer_input (window->buffer, 0);
+        gui_input_optimize_size (window->buffer);
+        gui_input_draw (window->buffer, 0);
         window->buffer->completion.position = -1;
     }
 }
@@ -464,7 +409,8 @@ void
 gui_action_transpose_chars (t_gui_window *window)
 {
     char *start, *prev_char, saved_char[4];
-    int size_current_char;
+    int size_current_char, size_start_char;
+    int pos_prev_char, pos_start;
     
     if (window->buffer->has_input)
     {
@@ -475,17 +421,26 @@ gui_action_transpose_chars (t_gui_window *window)
             
             start = utf8_add_offset (window->buffer->input_buffer,
                                      window->buffer->input_buffer_pos);
+            pos_start = start - window->buffer->input_buffer;
             prev_char = utf8_prev_char (window->buffer->input_buffer,
                                         start);
+            pos_prev_char = prev_char - window->buffer->input_buffer;
             size_current_char = start - prev_char;
+            size_start_char = utf8_char_size (start);
+            
             memcpy (saved_char, prev_char, size_current_char);
-            memcpy (prev_char, start, utf8_char_size (start));
-            start = utf8_next_char (prev_char);
+            memcpy (prev_char, start, size_start_char);
             memcpy (start, saved_char, size_current_char);
             
+            memcpy (saved_char, window->buffer->input_buffer_color_mask + pos_prev_char, size_current_char);
+            memcpy (window->buffer->input_buffer_color_mask + pos_prev_char,
+                    window->buffer->input_buffer_color_mask + pos_start, size_start_char);
+            memcpy (window->buffer->input_buffer_color_mask + pos_start,
+                    saved_char, size_current_char);
+            
             window->buffer->input_buffer_pos++;
             
-            gui_draw_buffer_input (window->buffer, 0);	
+            gui_input_draw (window->buffer, 0);	
             window->buffer->completion.position = -1;
         }
     }
@@ -503,7 +458,7 @@ gui_action_home (t_gui_window *window)
         if (window->buffer->input_buffer_pos > 0)
         {
             window->buffer->input_buffer_pos = 0;
-            gui_draw_buffer_input (window->buffer, 0);
+            gui_input_draw (window->buffer, 0);
         }
     }
 }
@@ -522,7 +477,7 @@ gui_action_end (t_gui_window *window)
         {
             window->buffer->input_buffer_pos =
                 window->buffer->input_buffer_length;
-            gui_draw_buffer_input (window->buffer, 0);
+            gui_input_draw (window->buffer, 0);
         }
     }
 }
@@ -539,7 +494,7 @@ gui_action_left (t_gui_window *window)
         if (window->buffer->input_buffer_pos > 0)
         {
             window->buffer->input_buffer_pos--;
-            gui_draw_buffer_input (window->buffer, 0);
+            gui_input_draw (window->buffer, 0);
         }
     }
 }
@@ -579,13 +534,13 @@ gui_action_previous_word (t_gui_window *window)
             else
                 window->buffer->input_buffer_pos = 0;
             
-            gui_draw_buffer_input (window->buffer, 0);
+            gui_input_draw (window->buffer, 0);
         }
     }
 }
 
 /*
- * gui_action_right: move to previous char
+ * gui_action_right: move to next char
  */
 
 void
@@ -597,7 +552,7 @@ gui_action_right (t_gui_window *window)
             window->buffer->input_buffer_length)
         {
             window->buffer->input_buffer_pos++;
-            gui_draw_buffer_input (window->buffer, 0);
+            gui_input_draw (window->buffer, 0);
         }
     }
 }
@@ -641,7 +596,7 @@ gui_action_next_word (t_gui_window *window)
                     utf8_pos (window->buffer->input_buffer,
                               utf8_prev_char (window->buffer->input_buffer, pos) - window->buffer->input_buffer);
             
-            gui_draw_buffer_input (window->buffer, 0);
+            gui_input_draw (window->buffer, 0);
         }
     }
 }
@@ -653,7 +608,7 @@ gui_action_next_word (t_gui_window *window)
 void
 gui_action_up (t_gui_window *window)
 {
-    if (window->buffer->dcc)
+    if (window->buffer->type == BUFFER_TYPE_DCC)
     {
         if (dcc_list)
         {
@@ -666,8 +621,8 @@ gui_action_up (t_gui_window *window)
                         ((t_irc_dcc *)(window->dcc_first))->prev_dcc;
                 window->dcc_selected =
                     ((t_irc_dcc *)(window->dcc_selected))->prev_dcc;
-                gui_draw_buffer_chat (window->buffer, 1);
-                gui_draw_buffer_input (window->buffer, 1);
+                gui_chat_draw (window->buffer, 1);
+                gui_input_draw (window->buffer, 1);
             }
         }
     }
@@ -692,6 +647,7 @@ gui_action_up (t_gui_window *window)
                 if (window->buffer->input_buffer_size > 0)
                 {
                     window->buffer->input_buffer[window->buffer->input_buffer_size] = '\0';
+                    window->buffer->input_buffer_color_mask[window->buffer->input_buffer_size] = '\0';
                     history_buffer_add (window->buffer, window->buffer->input_buffer);
                     history_global_add (window->buffer->input_buffer);
                 }
@@ -701,6 +657,7 @@ gui_action_up (t_gui_window *window)
                 if (window->buffer->input_buffer_size > 0)
                 {
                     window->buffer->input_buffer[window->buffer->input_buffer_size] = '\0';
+                    window->buffer->input_buffer_color_mask[window->buffer->input_buffer_size] = '\0';
                     if (window->buffer->ptr_history->prev_history->text)
 			free(window->buffer->ptr_history->prev_history->text);
                     window->buffer->ptr_history->prev_history->text = strdup (window->buffer->input_buffer);
@@ -710,13 +667,14 @@ gui_action_up (t_gui_window *window)
                 strlen (window->buffer->ptr_history->text);
             window->buffer->input_buffer_length =
                 utf8_strlen (window->buffer->ptr_history->text);
-            gui_optimize_input_buffer_size (window->buffer);
+            gui_input_optimize_size (window->buffer);
             window->buffer->input_buffer_pos =
                 window->buffer->input_buffer_length;
             window->buffer->input_buffer_1st_display = 0;
             strcpy (window->buffer->input_buffer,
                     window->buffer->ptr_history->text);
-            gui_draw_buffer_input (window->buffer, 0);
+            gui_input_init_color_mask (window->buffer);
+            gui_input_draw (window->buffer, 0);
         }
     }
 }
@@ -744,13 +702,14 @@ gui_action_up_global (t_gui_window *window)
                 strlen (history_global_ptr->text);
             window->buffer->input_buffer_length =
                 utf8_strlen (history_global_ptr->text);
-            gui_optimize_input_buffer_size (window->buffer);
+            gui_input_optimize_size (window->buffer);
             window->buffer->input_buffer_pos =
                 window->buffer->input_buffer_length;
             window->buffer->input_buffer_1st_display = 0;
             strcpy (window->buffer->input_buffer,
                     history_global_ptr->text);
-            gui_draw_buffer_input (window->buffer, 0);
+            gui_input_init_color_mask (window->buffer);
+            gui_input_draw (window->buffer, 0);
         }
     }
 }
@@ -762,7 +721,7 @@ gui_action_up_global (t_gui_window *window)
 void
 gui_action_down (t_gui_window *window)
 {
-    if (window->buffer->dcc)
+    if (window->buffer->type == BUFFER_TYPE_DCC)
     {
         if (dcc_list)
         {
@@ -786,8 +745,8 @@ gui_action_down (t_gui_window *window)
                 else
                     window->dcc_selected =
                         dcc_list->next_dcc;
-                gui_draw_buffer_chat (window->buffer, 1);
-                gui_draw_buffer_input (window->buffer, 1);
+                gui_chat_draw (window->buffer, 1);
+                gui_input_draw (window->buffer, 1);
             }
         }
     }
@@ -809,14 +768,17 @@ gui_action_down (t_gui_window *window)
                 window->buffer->input_buffer_size = 0;
                 window->buffer->input_buffer_length = 0;
             }
-            gui_optimize_input_buffer_size (window->buffer);
+            gui_input_optimize_size (window->buffer);
             window->buffer->input_buffer_pos =
                 window->buffer->input_buffer_length;
             window->buffer->input_buffer_1st_display = 0;
             if (window->buffer->ptr_history)
+            {
                 strcpy (window->buffer->input_buffer,
                         window->buffer->ptr_history->text);
-            gui_draw_buffer_input (window->buffer, 0);
+                gui_input_init_color_mask (window->buffer);
+            }
+            gui_input_draw (window->buffer, 0);
         }
     }
 }
@@ -845,14 +807,17 @@ gui_action_down_global (t_gui_window *window)
                 window->buffer->input_buffer_size = 0;
                 window->buffer->input_buffer_length = 0;
             }
-            gui_optimize_input_buffer_size (window->buffer);
+            gui_input_optimize_size (window->buffer);
             window->buffer->input_buffer_pos =
                 window->buffer->input_buffer_length;
             window->buffer->input_buffer_1st_display = 0;
             if (history_global_ptr)
+            {
                 strcpy (window->buffer->input_buffer,
                         history_global_ptr->text);
-            gui_draw_buffer_input (window->buffer, 0);
+                gui_input_init_color_mask (window->buffer);
+            }
+            gui_input_draw (window->buffer, 0);
         }
     }
 }
@@ -877,6 +842,46 @@ gui_action_page_down (t_gui_window *window)
     gui_window_page_down (window);
 }
 
+/*
+ * gui_action_scroll_up: display previous few lines in buffer
+ */
+
+void
+gui_action_scroll_up (t_gui_window *window)
+{
+    gui_window_scroll_up (window);
+}
+
+/*
+ * gui_action_scroll_down: display next few lines in buffer
+ */
+
+void
+gui_action_scroll_down (t_gui_window *window)
+{
+    gui_window_scroll_down (window);
+}
+
+/*
+ * gui_action_scroll_top: scroll to top of buffer
+ */
+
+void
+gui_action_scroll_top (t_gui_window *window)
+{
+    gui_window_scroll_top (window);
+}
+
+/*
+ * gui_action_scroll_bottom: scroll to bottom of buffer
+ */
+
+void
+gui_action_scroll_bottom (t_gui_window *window)
+{
+    gui_window_scroll_bottom (window);
+}
+
 /*
  * gui_action_nick_beginning: go to beginning of nicklist
  */
@@ -928,15 +933,15 @@ gui_action_jump_smart (t_gui_window *window)
     {
         if (!hotlist_initial_buffer)
             hotlist_initial_buffer = window->buffer;
-        gui_switch_to_buffer (window, hotlist->buffer);
-        gui_redraw_buffer (window->buffer);
+        gui_window_switch_to_buffer (window, hotlist->buffer);
+        gui_window_redraw_buffer (window->buffer);
     }
     else
     {
         if (hotlist_initial_buffer)
         {
-            gui_switch_to_buffer (window, hotlist_initial_buffer);
-            gui_redraw_buffer (window->buffer);
+            gui_window_switch_to_buffer (window, hotlist_initial_buffer);
+            gui_window_redraw_buffer (window->buffer);
             hotlist_initial_buffer = NULL;
         }
     }
@@ -949,22 +954,45 @@ gui_action_jump_smart (t_gui_window *window)
 void
 gui_action_jump_dcc (t_gui_window *window)
 {
-    if (window->buffer->dcc)
+    if (window->buffer->type == BUFFER_TYPE_DCC)
     {
-        if (buffer_before_dcc)
+        if (gui_buffer_before_dcc)
         {
-            gui_switch_to_buffer (window,
-                                  buffer_before_dcc);
-            gui_redraw_buffer (window->buffer);
+            gui_window_switch_to_buffer (window,
+                                         gui_buffer_before_dcc);
+            gui_window_redraw_buffer (window->buffer);
         }
     }
     else
     {
-        buffer_before_dcc = window->buffer;
+        gui_buffer_before_dcc = window->buffer;
         gui_buffer_switch_dcc (window);
     }
 }
 
+/*
+ * gui_action_jump_raw_data: jump to raw IRC data buffer
+ */
+
+void
+gui_action_jump_raw_data (t_gui_window *window)
+{
+    if (window->buffer->type == BUFFER_TYPE_RAW_DATA)
+    {
+        if (gui_buffer_before_raw_data)
+        {
+            gui_window_switch_to_buffer (window,
+                                         gui_buffer_before_raw_data);
+            gui_window_redraw_buffer (window->buffer);
+        }
+    }
+    else
+    {
+        gui_buffer_before_raw_data = window->buffer;
+        gui_buffer_switch_raw_data (window);
+    }
+}
+
 /*
  * gui_action_jump_last_buffer: jump to last buffer
  */
@@ -988,9 +1016,9 @@ gui_action_jump_server (t_gui_window *window)
         if (SERVER(window->buffer)->buffer !=
             window->buffer)
         {
-            gui_switch_to_buffer (window,
-                                  SERVER(window->buffer)->buffer);
-            gui_redraw_buffer (window->buffer);
+            gui_window_switch_to_buffer (window,
+                                         SERVER(window->buffer)->buffer);
+            gui_window_redraw_buffer (window->buffer);
         }
     }
 }
@@ -1031,8 +1059,8 @@ gui_action_jump_next_server (t_gui_window *window)
             if ((ptr_server->buffer == ptr_buffer)
                 && (ptr_buffer->all_servers))
                 ptr_buffer->server = ptr_server;
-            gui_switch_to_buffer (window, ptr_buffer);
-            gui_redraw_buffer (window->buffer);
+            gui_window_switch_to_buffer (window, ptr_buffer);
+            gui_window_redraw_buffer (window->buffer);
         }
     }
 }
@@ -1057,7 +1085,7 @@ gui_action_scroll_previous_highlight (t_gui_window *window)
 {
     t_gui_line *ptr_line;
     
-    if (!window->buffer->dcc)
+    if (window->buffer->type == BUFFER_TYPE_STANDARD)
     {
         if (window->buffer->lines)
         {
@@ -1071,8 +1099,8 @@ gui_action_scroll_previous_highlight (t_gui_window *window)
                     window->start_line_pos = 0;
                     window->first_line_displayed =
                         (window->start_line == window->buffer->lines);
-                    gui_draw_buffer_chat (window->buffer, 1);
-                    gui_draw_buffer_status (window->buffer, 0);
+                    gui_chat_draw (window->buffer, 1);
+                    gui_status_draw (window->buffer, 0);
                     return;
                 }
                 ptr_line = ptr_line->prev_line;
@@ -1090,7 +1118,7 @@ gui_action_scroll_next_highlight (t_gui_window *window)
 {
     t_gui_line *ptr_line;
     
-    if (!window->buffer->dcc)
+    if (window->buffer->type == BUFFER_TYPE_STANDARD)
     {
         if (window->buffer->lines)
         {
@@ -1104,8 +1132,8 @@ gui_action_scroll_next_highlight (t_gui_window *window)
                     window->start_line_pos = 0;
                     window->first_line_displayed =
                         (window->start_line == window->buffer->lines);
-                    gui_draw_buffer_chat (window->buffer, 1);
-                    gui_draw_buffer_status (window->buffer, 0);
+                    gui_chat_draw (window->buffer, 1);
+                    gui_status_draw (window->buffer, 0);
                     return;
                 }
                 ptr_line = ptr_line->next_line;
@@ -1123,7 +1151,7 @@ gui_action_scroll_unread (t_gui_window *window)
 {
     if (cfg_look_read_marker &&
         cfg_look_read_marker[0] &&
-        !window->buffer->dcc &&
+        (window->buffer->type == BUFFER_TYPE_STANDARD) &&
         window->buffer->last_read_line &&
         window->buffer->last_read_line != window->buffer->last_line)
     {
@@ -1131,8 +1159,8 @@ gui_action_scroll_unread (t_gui_window *window)
         window->start_line_pos = 0;
         window->first_line_displayed =
             (window->start_line == window->buffer->lines);
-        gui_draw_buffer_chat (window->buffer, 1);
-        gui_draw_buffer_status (window->buffer, 0);
+        gui_chat_draw (window->buffer, 1);
+        gui_status_draw (window->buffer, 0);
     }
 }
 
@@ -1146,7 +1174,7 @@ gui_action_hotlist_clear (t_gui_window *window)
     if (hotlist)
     {
         hotlist_free_all ();
-        gui_redraw_buffer (window->buffer);
+        gui_window_redraw_buffer (window->buffer);
     }
     hotlist_initial_buffer = window->buffer;
 }
@@ -1159,7 +1187,7 @@ void
 gui_action_infobar_clear (t_gui_window *window)
 {
     gui_infobar_remove ();
-    gui_draw_buffer_infobar (window->buffer, 1);
+    gui_infobar_draw (window->buffer, 1);
 }
 
 /*
@@ -1172,7 +1200,7 @@ gui_action_refresh_screen (t_gui_window *window)
     /* make gcc happy */
     (void) window;
     
-    gui_refresh_screen ();
+    gui_window_refresh_screen ();
 }
 
 /*
@@ -1183,5 +1211,5 @@ void
 gui_action_grab_key (t_gui_window *window)
 {
     if (window->buffer->has_input)
-        gui_key_init_grab ();
+        gui_keyboard_init_grab ();
 }
