@@ -30,12 +30,13 @@
 
 #include "../common/weechat.h"
 #include "irc.h"
+#include "../common/log.h"
 #include "../common/utf8.h"
 #include "../common/weeconfig.h"
 #include "../gui/gui.h"
 
 
-char *channel_modes = "iklmnst";
+char *channel_modes = "iklmnstp";
 
 
 /*
@@ -53,7 +54,7 @@ channel_new (t_irc_server *server, int channel_type, char *channel_name)
         fprintf (stderr, _("%s cannot allocate new channel"), WEECHAT_ERROR);
         return NULL;
     }
-
+    
     /* initialize new channel */
     new_channel->type = channel_type;
     new_channel->dcc_chat = NULL;
@@ -66,6 +67,9 @@ channel_new (t_irc_server *server, int channel_type, char *channel_name)
     new_channel->key = NULL;
     new_channel->nicks_count = 0;
     new_channel->checking_away = 0;
+    new_channel->away_message = NULL;
+    new_channel->cycle = 0;
+    new_channel->close = 0;
     new_channel->nicks = NULL;
     new_channel->last_nick = NULL;
 
@@ -95,6 +99,7 @@ channel_free (t_irc_server *server, t_irc_channel *channel)
     if ((t_irc_dcc *)(channel->dcc_chat) &&
         (!DCC_ENDED(((t_irc_dcc *)(channel->dcc_chat))->status)))
     {
+        ((t_irc_dcc *)(channel->dcc_chat))->channel = NULL;
         dcc_close ((t_irc_dcc *)(channel->dcc_chat), DCC_ABORTED);
         dcc_redraw (1);
     }
@@ -119,6 +124,8 @@ channel_free (t_irc_server *server, t_irc_channel *channel)
     if (channel->topic)
         free (channel->topic);
     nick_free_all (channel);
+    if (channel->away_message)
+        free (channel->away_message);
     free (channel);
     server->channels = new_channels;
 }
@@ -137,6 +144,7 @@ channel_free_all (t_irc_server *server)
 
 /*
  * channel_search: returns pointer on a channel with name
+ *                 WARNING: DCC chat channels are not returned by this function
  */
 
 t_irc_channel *
@@ -144,6 +152,28 @@ channel_search (t_irc_server *server, char *channel_name)
 {
     t_irc_channel *ptr_channel;
     
+    if (!server || !channel_name)
+        return NULL;
+    
+    for (ptr_channel = server->channels; ptr_channel;
+         ptr_channel = ptr_channel->next_channel)
+    {
+        if ((ptr_channel->type != CHANNEL_TYPE_DCC_CHAT)
+            && (ascii_strcasecmp (ptr_channel->name, channel_name) == 0))
+            return ptr_channel;
+    }
+    return NULL;
+}
+
+/*
+ * channel_search_any: returns pointer on a channel with name
+ */
+
+t_irc_channel *
+channel_search_any (t_irc_server *server, char *channel_name)
+{
+    t_irc_channel *ptr_channel;
+    
     if (!server || !channel_name)
         return NULL;
     
@@ -156,6 +186,51 @@ channel_search (t_irc_server *server, char *channel_name)
     return NULL;
 }
 
+/*
+ * channel_search_any_without_buffer: returns pointer on a channel with name
+ *                                    looks only for channels without buffer
+ */
+
+t_irc_channel *
+channel_search_any_without_buffer (t_irc_server *server, char *channel_name)
+{
+    t_irc_channel *ptr_channel;
+    
+    if (!server || !channel_name)
+        return NULL;
+    
+    for (ptr_channel = server->channels; ptr_channel;
+         ptr_channel = ptr_channel->next_channel)
+    {
+        if (!ptr_channel->buffer
+            && (ascii_strcasecmp (ptr_channel->name, channel_name) == 0))
+            return ptr_channel;
+    }
+    return NULL;
+}
+
+/*
+ * channel_search_dcc: returns pointer on a DCC chat channel with name
+ */
+
+t_irc_channel *
+channel_search_dcc (t_irc_server *server, char *channel_name)
+{
+    t_irc_channel *ptr_channel;
+    
+    if (!server || !channel_name)
+        return NULL;
+    
+    for (ptr_channel = server->channels; ptr_channel;
+         ptr_channel = ptr_channel->next_channel)
+    {
+        if ((ptr_channel->type == CHANNEL_TYPE_DCC_CHAT)
+            && (ascii_strcasecmp (ptr_channel->name, channel_name) == 0))
+            return ptr_channel;
+    }
+    return NULL;
+}
+
 /*
  * string_is_channel: returns 1 if string is channel
  */
@@ -325,7 +400,7 @@ channel_remove_away (t_irc_channel *channel)
         {
             NICK_SET_FLAG(ptr_nick, 0, NICK_AWAY);
         }
-        gui_draw_buffer_nick (channel->buffer, 0);
+        gui_nicklist_draw (channel->buffer, 0);
     }
 }
 
@@ -334,12 +409,18 @@ channel_remove_away (t_irc_channel *channel)
  */
 
 void
-channel_check_away (t_irc_server *server, t_irc_channel *channel)
+channel_check_away (t_irc_server *server, t_irc_channel *channel, int force)
 {
     if (channel->type == CHANNEL_TYPE_CHANNEL)
     {
-        channel->checking_away++;
-        server_sendf (server, "WHO %s\r\n", channel->name);
+        if (force || (cfg_irc_away_check_max_nicks == 0) ||
+            (channel->nicks_count <= cfg_irc_away_check_max_nicks))
+        {
+            channel->checking_away++;
+            server_sendf (server, "WHO %s\r\n", channel->name);
+        }
+        else
+            channel_remove_away (channel);
     }
 }
 
@@ -369,13 +450,16 @@ channel_create_dcc (t_irc_dcc *ptr_dcc)
 {
     t_irc_channel *ptr_channel;
     
-    ptr_channel = channel_search (ptr_dcc->server, ptr_dcc->nick);
+    ptr_channel = channel_search_dcc (ptr_dcc->server, ptr_dcc->nick);
     if (!ptr_channel)
-        ptr_channel = channel_new (ptr_dcc->server, CHANNEL_TYPE_PRIVATE,
+    {
+        ptr_channel = channel_new (ptr_dcc->server, CHANNEL_TYPE_DCC_CHAT,
                                    ptr_dcc->nick);
-    if (!ptr_channel)
-        return 0;
-    gui_buffer_new (gui_current_window, ptr_dcc->server, ptr_channel, 0, 0);
+        if (!ptr_channel)
+            return 0;
+        gui_buffer_new (gui_current_window, ptr_dcc->server, ptr_channel,
+                    BUFFER_TYPE_STANDARD, 0);
+    }
     
     if (ptr_channel->dcc_chat &&
         (!DCC_ENDED(((t_irc_dcc *)(ptr_channel->dcc_chat))->status)))
@@ -383,30 +467,10 @@ channel_create_dcc (t_irc_dcc *ptr_dcc)
     
     ptr_channel->dcc_chat = ptr_dcc;
     ptr_dcc->channel = ptr_channel;
-    gui_redraw_buffer (ptr_channel->buffer);
+    gui_window_redraw_buffer (ptr_channel->buffer);
     return 1;
 }
 
-/*
- * channel_remove_dcc: remove a DCC CHAT
- */
-
-void
-channel_remove_dcc (t_irc_dcc *ptr_dcc)
-{
-    t_irc_channel *ptr_channel;
-    
-    for (ptr_channel = ptr_dcc->server->channels; ptr_channel;
-        ptr_channel = ptr_channel->next_channel)
-    {
-        if ((t_irc_dcc *)(ptr_channel->dcc_chat) == ptr_dcc)
-        {
-            ptr_channel->dcc_chat = NULL;
-            gui_redraw_buffer (ptr_channel->buffer);
-        }
-    }
-}
-
 /*
  * channel_get_notify_level: get channel notify level
  */
@@ -414,7 +478,8 @@ channel_remove_dcc (t_irc_dcc *ptr_dcc)
 int
 channel_get_notify_level (t_irc_server *server, t_irc_channel *channel)
 {
-    char *name, *pos, *pos2, notify;
+    char *name, *pos, *pos2;
+    int notify;
     
     if ((!server) || (!channel))
         return NOTIFY_LEVEL_DEFAULT;
@@ -481,6 +546,9 @@ channel_print_log (t_irc_channel *channel)
     weechat_log_printf ("     limit. . . . : %d\n",     channel->limit);
     weechat_log_printf ("     key. . . . . : '%s'\n",   channel->key);
     weechat_log_printf ("     checking_away: %d\n",     channel->checking_away);
+    weechat_log_printf ("     away_message : '%s'\n",   channel->away_message);
+    weechat_log_printf ("     cycle. . . . : %d\n",     channel->cycle);
+    weechat_log_printf ("     close. . . . : %d\n",     channel->close);
     weechat_log_printf ("     nicks. . . . : 0x%X\n",   channel->nicks);
     weechat_log_printf ("     last_nick. . : 0x%X\n",   channel->last_nick);
     weechat_log_printf ("     buffer . . . : 0x%X\n",   channel->buffer);
