@@ -39,8 +39,9 @@
 
 #include "../common/weechat.h"
 #include "irc.h"
-#include "../common/weeconfig.h"
+#include "../common/log.h"
 #include "../common/hotlist.h"
+#include "../common/weeconfig.h"
 #include "../gui/gui.h"
 
 
@@ -57,11 +58,11 @@ char *dcc_status_string[] =     /* strings for DCC status                   */
 void
 dcc_redraw (int highlight)
 {
-    gui_redraw_buffer (gui_get_dcc_buffer (gui_current_window));
+    gui_window_redraw_buffer (gui_buffer_get_dcc (gui_current_window));
     if (highlight)
     {
-        hotlist_add (highlight, NULL, gui_get_dcc_buffer (gui_current_window));
-        gui_draw_buffer_status (gui_current_window->buffer, 0);
+        hotlist_add (highlight, NULL, gui_buffer_get_dcc (gui_current_window));
+        gui_status_draw (gui_current_window->buffer, 0);
     }
 }
 
@@ -146,34 +147,39 @@ dcc_file_is_resumable (t_irc_dcc *ptr_dcc, char *filename)
 void
 dcc_find_filename (t_irc_dcc *ptr_dcc)
 {
-    char *ptr_home, *filename2;
+    char *dir1, *dir2, *filename2;
     
     if (!DCC_IS_FILE(ptr_dcc->type))
         return;
     
-    ptr_home = getenv ("HOME");
-    ptr_dcc->local_filename = (char *) malloc (strlen (cfg_dcc_download_path) +
+    dir1 = weechat_strreplace (cfg_dcc_download_path, "~", getenv ("HOME"));
+    if (!dir1)
+        return;
+    dir2 = weechat_strreplace (dir1, "%h", weechat_home);
+    if (!dir2)
+    {
+        free (dir1);
+        return;
+    }
+    
+    ptr_dcc->local_filename = (char *) malloc (strlen (dir2) +
                                                strlen (ptr_dcc->nick) +
-                                               strlen (ptr_dcc->filename) +
-                                               ((cfg_dcc_download_path[0] == '~') ?
-                                                strlen (ptr_home) : 0) +
-                                               4);
+                                               strlen (ptr_dcc->filename) + 4);
     if (!ptr_dcc->local_filename)
         return;
     
-    if (cfg_dcc_download_path[0] == '~')
-    {
-        strcpy (ptr_dcc->local_filename, ptr_home);
-        strcat (ptr_dcc->local_filename, cfg_dcc_download_path + 1);
-    }
-    else
-        strcpy (ptr_dcc->local_filename, cfg_dcc_download_path);
+    strcpy (ptr_dcc->local_filename, dir2);
     if (ptr_dcc->local_filename[strlen (ptr_dcc->local_filename) - 1] != DIR_SEPARATOR_CHAR)
         strcat (ptr_dcc->local_filename, DIR_SEPARATOR);
     strcat (ptr_dcc->local_filename, ptr_dcc->nick);
     strcat (ptr_dcc->local_filename, ".");
     strcat (ptr_dcc->local_filename, ptr_dcc->filename);
     
+    if (dir1)
+        free (dir1);
+    if (dir2 )
+        free (dir2);
+    
     /* file already exists? */
     if (access (ptr_dcc->local_filename, F_OK) == 0)
     {
@@ -278,10 +284,10 @@ dcc_connect (t_irc_dcc *ptr_dcc)
     else
         ptr_dcc->status = DCC_CONNECTING;
     
-    if (ptr_dcc->sock == -1)
+    if (ptr_dcc->sock < 0)
     {
         ptr_dcc->sock = socket (AF_INET, SOCK_STREAM, 0);
-        if (ptr_dcc->sock == -1)
+        if (ptr_dcc->sock < 0)
             return 0;
     }
 
@@ -339,6 +345,20 @@ dcc_free (t_irc_dcc *ptr_dcc)
 {
     t_irc_dcc *new_dcc_list;
     
+    /* DCC CHAT with channel => remove channel
+       (to prevent channel from becoming standard pv) */
+    if (ptr_dcc->channel)
+    {
+        /* check if channel is used for another active DCC CHAT */
+        if (!ptr_dcc->channel->dcc_chat
+            || (DCC_ENDED(((t_irc_dcc *)(ptr_dcc->channel->dcc_chat))->status)))
+        {
+            gui_buffer_free (ptr_dcc->channel->buffer, 1);
+            channel_free (ptr_dcc->server, ptr_dcc->channel);
+        }
+    }
+
+    /* remove DCC from list */
     if (ptr_dcc->prev_dcc)
     {
         (ptr_dcc->prev_dcc)->next_dcc = ptr_dcc->next_dcc;
@@ -346,10 +366,10 @@ dcc_free (t_irc_dcc *ptr_dcc)
     }
     else
         new_dcc_list = ptr_dcc->next_dcc;
-    
     if (ptr_dcc->next_dcc)
         (ptr_dcc->next_dcc)->prev_dcc = ptr_dcc->prev_dcc;
-    
+
+    /* free data */
     if (ptr_dcc->nick)
         free (ptr_dcc->nick);
     if (ptr_dcc->unterminated_message)
@@ -439,20 +459,15 @@ dcc_close (t_irc_dcc *ptr_dcc, int status)
         }
     }
     
-    ptr_dcc->channel = NULL;
-    
-    if (DCC_IS_CHAT(ptr_dcc->type))
-        channel_remove_dcc (ptr_dcc);
-    
     if (DCC_IS_FILE(ptr_dcc->type))
         dcc_calculate_speed (ptr_dcc, 1);
     
-    if (ptr_dcc->sock != -1)
+    if (ptr_dcc->sock >= 0)
     {
         close (ptr_dcc->sock);
         ptr_dcc->sock = -1;
     }
-    if (ptr_dcc->file != -1)
+    if (ptr_dcc->file >= 0)
     {
         close (ptr_dcc->file);
         ptr_dcc->file = -1;
@@ -495,6 +510,25 @@ dcc_channel_for_chat (t_irc_dcc *ptr_dcc)
                      GUI_COLOR(COLOR_WIN_CHAT));
 }
 
+/*
+ * dcc_chat_remove_channel: remove a buffer for DCC chat
+ */
+
+void
+dcc_chat_remove_channel (t_irc_channel *channel)
+{
+    t_irc_dcc *ptr_dcc;
+    
+    if (!channel)
+        return;
+
+    for (ptr_dcc = dcc_list; ptr_dcc; ptr_dcc = ptr_dcc->next_dcc)
+    {
+        if (ptr_dcc->channel == channel)
+            ptr_dcc->channel = NULL;
+    }
+}
+
 /*
  * dcc_recv_connect_init: connect to sender and init file or chat
  */
@@ -835,7 +869,7 @@ dcc_add (t_irc_server *server, int type, unsigned long addr, int port, char *nic
         dcc_accept (new_dcc);
     else
         dcc_redraw (HOTLIST_PRIVATE);
-    gui_draw_buffer_status (gui_current_window->buffer, 0);
+    gui_status_draw (gui_current_window->buffer, 0);
     
     return new_dcc;
 }
@@ -847,7 +881,7 @@ dcc_add (t_irc_server *server, int type, unsigned long addr, int port, char *nic
 void
 dcc_send_request (t_irc_server *server, int type, char *nick, char *filename)
 {
-    char *ptr_home, *filename2, *short_filename, *pos;
+    char *dir1, *dir2, *filename2, *short_filename, *pos;
     int spaces, args, port_start, port_end;
     struct stat st;
     int sock, port;
@@ -872,12 +906,17 @@ dcc_send_request (t_irc_server *server, int type, char *nick, char *filename)
             filename2 = strdup (filename);
         else
         {
-            ptr_home = getenv ("HOME");
-            filename2 = (char *) malloc (strlen (cfg_dcc_upload_path) +
-                                         strlen (filename) +
-                                         ((cfg_dcc_upload_path[0] == '~') ?
-                                             strlen (ptr_home) : 0) +
-                                         4);
+            dir1 = weechat_strreplace (cfg_dcc_upload_path, "~", getenv ("HOME"));
+            if (!dir1)
+                return;
+            dir2 = weechat_strreplace (dir1, "%h", weechat_home);
+            if (!dir2)
+            {
+                free (dir1);
+                return;
+            }
+            filename2 = (char *) malloc (strlen (dir2) +
+                                         strlen (filename) + 4);
             if (!filename2)
             {
                 irc_display_prefix (server, server->buffer, PREFIX_ERROR);
@@ -886,16 +925,14 @@ dcc_send_request (t_irc_server *server, int type, char *nick, char *filename)
                             WEECHAT_ERROR);
                 return;
             }
-            if (cfg_dcc_upload_path[0] == '~')
-            {
-                strcpy (filename2, ptr_home);
-                strcat (filename2, cfg_dcc_upload_path + 1);
-            }
-            else
-                strcpy (filename2, cfg_dcc_upload_path);
+            strcpy (filename2, dir2);
             if (filename2[strlen (filename2) - 1] != DIR_SEPARATOR_CHAR)
                 strcat (filename2, DIR_SEPARATOR);
             strcat (filename2, filename);
+            if (dir1)
+                free (dir1);
+            if (dir2)
+                free (dir2);
         }
 #endif
         
@@ -942,7 +979,7 @@ dcc_send_request (t_irc_server *server, int type, char *nick, char *filename)
     
     /* open socket for DCC */
     sock = socket (AF_INET, SOCK_STREAM, 0);
-    if (sock == -1)
+    if (sock < 0)
     {
         irc_display_prefix (server, server->buffer, PREFIX_ERROR);
         gui_printf (server->buffer,
@@ -1102,7 +1139,7 @@ dcc_chat_sendf (t_irc_dcc *ptr_dcc, char *fmt, ...)
     char *buf2;
     int size_buf;
 
-    if (!ptr_dcc || (ptr_dcc->sock == -1))
+    if (!ptr_dcc || (ptr_dcc->sock < 0))
         return;
     
     va_start (args, fmt);
@@ -1201,30 +1238,25 @@ dcc_chat_recv (t_irc_dcc *ptr_dcc)
                 ptr_buf_color = (char *)gui_color_decode ((ptr_buf2) ?
                                                           (unsigned char *)ptr_buf2 : (unsigned char *)ptr_buf,
                                                           cfg_irc_colors_receive);
-                gui_printf_type (ptr_dcc->channel->buffer, MSG_TYPE_NICK,
-                                 "%s<", GUI_COLOR(COLOR_WIN_CHAT_DARK));
+                
                 if (irc_is_highlight (ptr_buf, ptr_dcc->server->nick))
                 {
-                    gui_printf_type (ptr_dcc->channel->buffer,
-                                     MSG_TYPE_NICK | MSG_TYPE_HIGHLIGHT,
-                                     "%s%s",
-                                     GUI_COLOR(COLOR_WIN_CHAT_HIGHLIGHT),
-                                     ptr_dcc->nick);
-                    if ( (cfg_look_infobar_delay_highlight > 0)
-                        && (ptr_dcc->channel->buffer != gui_current_window->buffer) )
+                    irc_display_nick (ptr_dcc->channel->buffer, NULL, ptr_dcc->nick,
+                                      MSG_TYPE_NICK | MSG_TYPE_HIGHLIGHT, 1,
+                                      COLOR_WIN_CHAT_HIGHLIGHT, 0);
+                    if ((cfg_look_infobar_delay_highlight > 0)
+                        && (ptr_dcc->channel->buffer != gui_current_window->buffer))
+                    {
                         gui_infobar_printf (cfg_look_infobar_delay_highlight,
                                             COLOR_WIN_INFOBAR_HIGHLIGHT,
                                             _("Private %s> %s"),
                                             ptr_dcc->nick,
                                             (ptr_buf_color) ? ptr_buf_color : ((ptr_buf2) ? ptr_buf2 : ptr_buf));
+                    }
                 }
                 else
-                    gui_printf_type (ptr_dcc->channel->buffer, MSG_TYPE_NICK,
-                                     "%s%s",
-                                     GUI_COLOR(COLOR_WIN_NICK_PRIVATE),
-                                     ptr_dcc->nick);
-                gui_printf_type (ptr_dcc->channel->buffer, MSG_TYPE_NICK,
-                                 "%s> ", GUI_COLOR(COLOR_WIN_CHAT_DARK));
+                    irc_display_nick (ptr_dcc->channel->buffer, NULL, ptr_dcc->nick,
+                                      MSG_TYPE_NICK, 1, COLOR_WIN_NICK_PRIVATE, 0);
                 gui_printf_type (ptr_dcc->channel->buffer, MSG_TYPE_MSG,
                                  "%s%s\n",
                                  GUI_COLOR(COLOR_WIN_CHAT),
@@ -1485,7 +1517,7 @@ dcc_end ()
     
     for (ptr_dcc = dcc_list; ptr_dcc; ptr_dcc = ptr_dcc->next_dcc)
     {
-        if (ptr_dcc->sock != -1)
+        if (ptr_dcc->sock >= 0)
         {
             if (ptr_dcc->status == DCC_ACTIVE)
                 weechat_log_printf (_("Aborting active DCC: \"%s\" from %s\n"),
