@@ -35,6 +35,54 @@
 #include "../gui/gui.h"
 
 
+/*
+ * irc_find_context: find window/buffer for a server/channel
+ */
+
+void
+irc_find_context (t_irc_server *server, t_irc_channel *channel,
+                  t_gui_window **window, t_gui_buffer **buffer)
+{
+    t_gui_window *ptr_win;
+    
+    if (!buffer)
+        return;
+    
+    /* first find buffer */
+    *buffer = NULL;
+    if (channel && channel->buffer)
+        *buffer = channel->buffer;
+    else
+    {
+        if (server && server->buffer)
+            *buffer = server->buffer;
+        else
+            *buffer = gui_current_window->buffer;
+    }
+    
+    /* then find first window displaying this buffer */
+    if (window)
+    {
+        *window = NULL;
+        if (gui_current_window->buffer == *buffer)
+            *window = gui_current_window;
+        else
+        {
+            for (ptr_win = gui_windows; ptr_win;
+                 ptr_win = ptr_win->next_window)
+            {
+                if (ptr_win->buffer == *buffer)
+                {
+                    *window = ptr_win;
+                    break;
+                }
+            }
+            if (!*window)
+                *window = gui_current_window;
+        }
+    }
+}
+
 /*
  * irc_display_prefix: display a prefix for action/info/error msg
  *                     prefix must be 3 chars length
@@ -44,12 +92,23 @@ void
 irc_display_prefix (t_irc_server *server, t_gui_buffer *buffer, char *prefix)
 {
     int type;
+    char format[32];
     
     type = MSG_TYPE_INFO | MSG_TYPE_PREFIX;
     
     if (!cfg_log_plugin_msg && (prefix == PREFIX_PLUGIN))
         type |= MSG_TYPE_NOLOG;
     
+    if (buffer)
+    {
+        if (cfg_look_align_other
+            && (BUFFER_IS_CHANNEL(buffer) || BUFFER_IS_PRIVATE(buffer)))
+        {
+            snprintf (format, 32, "%%-%ds", cfg_look_align_size - 2);
+            gui_printf_type (buffer, MSG_TYPE_NICK, format, " ");
+        }
+    }
+    
     if (prefix[0] == prefix[2])
     {
         gui_printf_type (buffer, type, "%s%c%s%c%s%c ",
@@ -84,17 +143,74 @@ irc_display_prefix (t_irc_server *server, t_gui_buffer *buffer, char *prefix)
 
 /*
  * irc_display_nick: display nick in chat window
- *                   if color_nick < 0 then nick is highlighted
  */
 
 void
 irc_display_nick (t_gui_buffer *buffer, t_irc_nick *nick, char *nickname,
-                  int type, int display_around, int color_nick, int no_nickmode)
+                  int type, int display_around, int force_color, int no_nickmode)
 {
-    if (display_around)
+    char format[32], *ptr_nickname;
+    int max_align, i, nickname_length, external_nick, length, spaces;
+    int disable_prefix_suffix;
+
+    max_align = (cfg_look_align_size_max >= cfg_look_align_size) ?
+        cfg_look_align_size_max : cfg_look_align_size;
+    
+    ptr_nickname = strdup ((nick) ? nick->nick : nickname);
+    if (!ptr_nickname)
+        return;
+    nickname_length = strlen (ptr_nickname); 
+    external_nick = (!nick && !BUFFER_IS_PRIVATE(buffer));
+    disable_prefix_suffix = ((cfg_look_align_nick != CFG_LOOK_ALIGN_NICK_NONE)
+                             && ((int)strlen (cfg_look_nick_prefix) +
+                                 (int)strlen (cfg_look_nick_suffix) > max_align - 4));
+    
+    /* calculate length to display, to truncate it if too long */
+    length = nickname_length;
+    if (!disable_prefix_suffix && cfg_look_nick_prefix)
+        length += strlen (cfg_look_nick_prefix);
+    if (external_nick)
+        length += 2;
+    if (nick)
+    {
+        if (nick->flags & (NICK_CHANOWNER | NICK_CHANADMIN |
+                           NICK_OP | NICK_HALFOP | NICK_VOICE))
+            length += 1;
+        else if (cfg_look_nickmode_empty && !no_nickmode)
+            length += 1;
+    }
+    if (!disable_prefix_suffix && cfg_look_nick_suffix)
+        length += strlen (cfg_look_nick_suffix);
+    
+    /* calculate number of spaces to insert before or after nick */
+    spaces = 0;
+    if (cfg_look_align_nick != CFG_LOOK_ALIGN_NICK_NONE)
+    {
+        if (length > max_align)
+            spaces = max_align - length;
+        else if (length > cfg_look_align_size)
+            spaces = 0;
+        else
+            spaces = cfg_look_align_size - length;
+    }
+    
+    /* display prefix */
+    if (display_around && !disable_prefix_suffix
+        && cfg_look_nick_prefix && cfg_look_nick_prefix[0])
         gui_printf_type (buffer, type, "%s%s",
                          GUI_COLOR(COLOR_WIN_CHAT_DARK),
-                         (nick || BUFFER_IS_PRIVATE(buffer)) ? "<" : ">");
+                         cfg_look_nick_prefix);
+    
+    /* display spaces before nick, if needed */
+    if (display_around
+        && (cfg_look_align_nick == CFG_LOOK_ALIGN_NICK_RIGHT)
+        && (spaces > 0))
+    {
+        snprintf (format, 32, "%%-%ds", spaces);
+        gui_printf_type (buffer, type, format, " ");
+    }
+    
+    /* display nick mode */
     if (nick && cfg_look_nickmode)
     {
         if (nick->flags & NICK_CHANOWNER)
@@ -112,26 +228,71 @@ irc_display_nick (t_gui_buffer *buffer, t_irc_nick *nick, char *nickname,
         else if (nick->flags & NICK_VOICE)
             gui_printf_type (buffer, type, "%s+",
                              GUI_COLOR(COLOR_WIN_NICK_VOICE));
-        else
-            if (cfg_look_nickmode_empty && !no_nickmode)
-                gui_printf_type (buffer, type, "%s ",
-                                 GUI_COLOR(COLOR_WIN_CHAT));
+        else if (cfg_look_nickmode_empty && !no_nickmode)
+            gui_printf_type (buffer, type, "%s ",
+                             GUI_COLOR(COLOR_WIN_CHAT));
     }
-    if (color_nick < 0)
-        gui_printf_type (buffer, type, "%s%s",
-                         GUI_COLOR(COLOR_WIN_CHAT_HIGHLIGHT),
-                         (nick) ? nick->nick : nickname);
-    else
-        gui_printf_type (buffer, type, "%s%s",
-                         GUI_COLOR((nick && color_nick) ?
-                                   nick->color : COLOR_WIN_CHAT),
-                         (nick) ? nick->nick : nickname);
     
-    if (display_around)
+    /* display nick */
+    if (external_nick)
         gui_printf_type (buffer, type, "%s%s",
                          GUI_COLOR(COLOR_WIN_CHAT_DARK),
-                         (nick || BUFFER_IS_PRIVATE(buffer)) ? "> " : "< ");
-    gui_printf_type (buffer, type, GUI_NO_COLOR);
+                         "(");
+    if (display_around && (spaces < 0))
+    {
+        i = nickname_length + spaces - 1;
+        if (i < 3)
+        {
+            if (nickname_length < 3)
+                i = nickname_length;
+            else
+                i = 3;
+        }
+        ptr_nickname[i] = '\0';
+    }
+    if (display_around)
+        gui_printf_type_nick (buffer, type,
+                              (nick) ? nick->nick : nickname,
+                              "%s%s",
+                              (force_color >= 0) ?
+                              GUI_COLOR(force_color) :
+                              GUI_COLOR((nick) ? nick->color : COLOR_WIN_CHAT),
+                              ptr_nickname);
+    else
+        gui_printf_type (buffer, type,
+                         "%s%s",
+                         (force_color >= 0) ?
+                         GUI_COLOR(force_color) :
+                         GUI_COLOR((nick) ? nick->color : COLOR_WIN_CHAT),
+                         ptr_nickname);
+    if (display_around && (spaces < 0))
+        gui_printf_type (buffer, type, "%s+",
+                         GUI_COLOR(COLOR_WIN_NICK_MORE));
+    if (external_nick)
+        gui_printf_type (buffer, type, "%s%s",
+                         GUI_COLOR(COLOR_WIN_CHAT_DARK),
+                         ")");
+    
+    /* display spaces after nick, if needed */
+    if (display_around
+        && (cfg_look_align_nick == CFG_LOOK_ALIGN_NICK_LEFT)
+        && (spaces > 0))
+    {
+        snprintf (format, 32, "%%-%ds", spaces);
+        gui_printf_type (buffer, type, format, " ");
+    }
+    
+    /* display suffix */
+    if (display_around && !disable_prefix_suffix
+        && cfg_look_nick_suffix && cfg_look_nick_suffix[0])
+        gui_printf_type (buffer, type, "%s%s",
+                         GUI_COLOR(COLOR_WIN_CHAT_DARK),
+                         cfg_look_nick_suffix);
+    
+    gui_printf_type (buffer, type, "%s%s",
+                     GUI_NO_COLOR,
+                     (display_around) ? " " : "");
+    free (ptr_nickname);
 }
 
 /*
@@ -142,12 +303,19 @@ void
 irc_display_away (t_irc_server *server, char *string1, char *string2)
 {
     t_irc_channel *ptr_channel;
+    char format[32];
     
     for (ptr_channel = server->channels; ptr_channel;
          ptr_channel = ptr_channel->next_channel)
     {
         if (ptr_channel->type == CHANNEL_TYPE_CHANNEL)
         {
+            if (cfg_look_align_other)
+            {
+                snprintf (format, 32, "%%-%ds", cfg_look_align_size + 1);
+                gui_printf_type (ptr_channel->buffer, MSG_TYPE_NICK,
+                                 format, " ");
+            }
             gui_printf_nolog (ptr_channel->buffer,
                               "%s[%s%s%s %s: %s%s]\n",
                               GUI_COLOR(COLOR_WIN_CHAT_DARK),
@@ -167,14 +335,16 @@ irc_display_away (t_irc_server *server, char *string1, char *string2)
 
 void
 irc_display_mode (t_irc_server *server, t_gui_buffer *buffer,
-                  char *channel_name, char set_flag,
+                  char *channel_name, char *nick_name, char set_flag,
                   char *symbol, char *nick_host, char *message, char *param)
 {
     irc_display_prefix (server, buffer, PREFIX_INFO);
     gui_printf (buffer, "%s[%s%s%s/%s%c%s%s] %s%s",
                 GUI_COLOR(COLOR_WIN_CHAT_DARK),
-                GUI_COLOR(COLOR_WIN_CHAT_CHANNEL),
-                channel_name,
+                (channel_name) ?
+                 GUI_COLOR(COLOR_WIN_CHAT_CHANNEL) :
+                 GUI_COLOR(COLOR_WIN_CHAT_NICK),
+                (channel_name) ? channel_name : nick_name,
                 GUI_COLOR(COLOR_WIN_CHAT),
                 GUI_COLOR(COLOR_WIN_CHAT_CHANNEL),
                 set_flag,
@@ -243,7 +413,9 @@ irc_display_server (t_irc_server *server)
     gui_printf (NULL, "  server_username  . . . . . : %s\n",
                 server->username);
     gui_printf (NULL, "  server_realname  . . . . . : %s\n",
-               server->realname);
+                server->realname);
+    gui_printf (NULL, "  server_hostname  . . . . . : %s\n",
+                (server->hostname) ? server->hostname : "");
     gui_printf (NULL, "  server_command . . . . . . : %s\n",
                 (server->command && server->command[0]) ?
                 server->command : "");
