@@ -30,6 +30,7 @@
 
 #include "../common/weechat.h"
 #include "irc.h"
+#include "../common/log.h"
 #include "../common/weeconfig.h"
 
 
@@ -163,6 +164,27 @@ nick_insert_sorted (t_irc_channel *channel, t_irc_nick *nick)
     }
 }
 
+/*
+ * nick_resort: resort nick in the list
+ */
+
+void
+nick_resort (t_irc_channel *channel, t_irc_nick *nick)
+{
+    /* temporarly remove nick from list */
+    if (nick == channel->nicks)
+        channel->nicks = nick->next_nick;
+    else
+        nick->prev_nick->next_nick = nick->next_nick;
+    if (nick->next_nick)
+        nick->next_nick->prev_nick = nick->prev_nick;
+    if (nick == channel->last_nick)
+        channel->last_nick = nick->prev_nick;
+    
+    /* insert again nick into sorted list */
+    nick_insert_sorted (channel, nick);
+}
+
 /*
  * nick_new: allocate a new nick for a channel and add it to the nick list
  */
@@ -183,6 +205,7 @@ nick_new (t_irc_server *server, t_irc_channel *channel, char *nick_name,
         NICK_SET_FLAG(new_nick, is_op, NICK_OP);
         NICK_SET_FLAG(new_nick, is_halfop, NICK_HALFOP);
         NICK_SET_FLAG(new_nick, has_voice, NICK_VOICE);
+        nick_resort (channel, new_nick);
         return new_nick;
     }
     
@@ -192,6 +215,7 @@ nick_new (t_irc_server *server, t_irc_channel *channel, char *nick_name,
     
     /* initialize new nick */
     new_nick->nick = strdup (nick_name);
+    new_nick->host = NULL;
     new_nick->flags = 0;
     NICK_SET_FLAG(new_nick, is_chanowner, NICK_CHANOWNER);
     NICK_SET_FLAG(new_nick, is_chanadmin, NICK_CHANADMIN);
@@ -211,27 +235,6 @@ nick_new (t_irc_server *server, t_irc_channel *channel, char *nick_name,
     return new_nick;
 }
 
-/*
- * nick_resort: resort nick in the list
- */
-
-void
-nick_resort (t_irc_channel *channel, t_irc_nick *nick)
-{
-    /* temporarly remove nick from list */
-    if (nick == channel->nicks)
-        channel->nicks = nick->next_nick;
-    else
-        nick->prev_nick->next_nick = nick->next_nick;
-    if (nick->next_nick)
-        nick->next_nick->prev_nick = nick->prev_nick;
-    if (nick == channel->last_nick)
-        channel->last_nick = nick->prev_nick;
-    
-    /* insert again nick into sorted list */
-    nick_insert_sorted (channel, nick);
-}
-
 /*
  * nick_change: change nickname and move it if necessary (list is sorted)
  */
@@ -278,12 +281,14 @@ nick_free (t_irc_channel *channel, t_irc_nick *nick)
     
     if (nick->next_nick)
         (nick->next_nick)->prev_nick = nick->prev_nick;
-
+    
     channel->nicks_count--;
     
     /* free data */
     if (nick->nick)
         free (nick->nick);
+    if (nick->host)
+        free (nick->host);
     free (nick);
     channel->nicks = new_nicks;
 }
@@ -389,11 +394,16 @@ nick_get_max_length (t_irc_channel *channel)
 void
 nick_set_away (t_irc_channel *channel, t_irc_nick *nick, int is_away)
 {
-    if (((is_away) && (!(nick->flags & NICK_AWAY))) ||
-        ((!is_away) && (nick->flags & NICK_AWAY)))
+    if ((cfg_irc_away_check > 0)
+        && ((cfg_irc_away_check_max_nicks == 0) ||
+            (channel->nicks_count <= cfg_irc_away_check_max_nicks)))
     {
-        NICK_SET_FLAG(nick, is_away, NICK_AWAY);
-        gui_draw_buffer_nick (channel->buffer, 0);
+        if (((is_away) && (!(nick->flags & NICK_AWAY))) ||
+            ((!is_away) && (nick->flags & NICK_AWAY)))
+        {
+            NICK_SET_FLAG(nick, is_away, NICK_AWAY);
+            gui_nicklist_draw (channel->buffer, 0);
+        }
     }
 }
 
@@ -405,6 +415,7 @@ void
 nick_print_log (t_irc_nick *nick)
 {
     weechat_log_printf ("=> nick %s (addr:0x%X)]\n",    nick->nick, nick);
+    weechat_log_printf ("     host . . . . . : %s\n",   nick->host);
     weechat_log_printf ("     flags. . . . . : %d\n",   nick->flags);
     weechat_log_printf ("     color. . . . . : %d\n",   nick->color);
     weechat_log_printf ("     prev_nick. . . : 0x%X\n", nick->prev_nick);
