@@ -45,6 +45,7 @@
 
 #include "../common/weechat.h"
 #include "irc.h"
+#include "../common/log.h"
 #include "../common/weeconfig.h"
 #include "../gui/gui.h"
 
@@ -56,6 +57,9 @@ t_irc_message *recv_msgq, *msgq_last_msg;
 
 int check_away = 0;
 
+char *nick_modes = "aiwroOs";
+
+
 /*
  * server_init: init server struct with default values
  */
@@ -79,6 +83,7 @@ server_init (t_irc_server *server)
     server->nick3 = NULL;
     server->username = NULL;
     server->realname = NULL;
+    server->hostname = NULL;
     server->command = NULL;
     server->command_delay = 1;
     server->autojoin = NULL;
@@ -97,6 +102,9 @@ server_init (t_irc_server *server)
     server->ssl_connected = 0;
     server->unterminated_message = NULL;
     server->nick = NULL;
+    server->nick_modes = (char *) malloc (NUM_NICK_MODES + 1);
+    memset (server->nick_modes, ' ', NUM_NICK_MODES);
+    server->nick_modes[NUM_NICK_MODES] = '\0';
     server->reconnect_start = 0;
     server->reconnect_join = 0;
     server->is_away = 0;
@@ -105,6 +113,7 @@ server_init (t_irc_server *server)
     server->lag_check_time.tv_sec = 0;
     server->lag_check_time.tv_usec = 0;
     server->lag_next_check = time (NULL) + cfg_irc_lag_check;
+    server->cmd_list_regexp = NULL;
     server->buffer = NULL;
     server->saved_buffer = NULL;
     server->channels = NULL;
@@ -291,6 +300,8 @@ server_destroy (t_irc_server *server)
         free (server->username);
     if (server->realname)
         free (server->realname);
+    if (server->hostname)
+        free (server->hostname);
     if (server->command)
         free (server->command);
     if (server->autojoin)
@@ -307,6 +318,8 @@ server_destroy (t_irc_server *server)
         free (server->unterminated_message);
     if (server->nick)
         free (server->nick);
+    if (server->nick_modes)
+        free (server->nick_modes);
     if (server->channels)
         channel_free_all (server);
 }
@@ -360,12 +373,14 @@ server_free_all ()
  */
 
 t_irc_server *
-server_new (char *name, int autoconnect, int autoreconnect, int autoreconnect_delay,
-            int command_line, char *address, int port, int ipv6, int ssl, char *password,
+server_new (char *name, int autoconnect, int autoreconnect,
+            int autoreconnect_delay, int command_line, char *address,
+            int port, int ipv6, int ssl, char *password,
             char *nick1, char *nick2, char *nick3, char *username,
-            char *realname, char *command, int command_delay, char *autojoin,
-            int autorejoin, char *notify_levels, char *charset_decode_iso,
-            char *charset_decode_utf, char *charset_encode)
+            char *realname, char *hostname, char *command, int command_delay,
+            char *autojoin, int autorejoin, char *notify_levels,
+            char *charset_decode_iso, char *charset_decode_utf,
+            char *charset_encode)
 {
     t_irc_server *new_server;
     
@@ -375,13 +390,14 @@ server_new (char *name, int autoconnect, int autoreconnect, int autoreconnect_de
 #ifdef DEBUG
     weechat_log_printf ("Creating new server (name:%s, address:%s, port:%d, pwd:%s, "
                         "nick1:%s, nick2:%s, nick3:%s, username:%s, realname:%s, "
-                        "command:%s, autojoin:%s, autorejoin:%s, notify_levels:%s, "
-                        "decode_iso:%s, decode_utf:%s, encode:%s)\n",
+                        "hostname: %s, command:%s, autojoin:%s, autorejoin:%s, "
+                        "notify_levels:%s, decode_iso:%s, decode_utf:%s, encode:%s)\n",
                         name, address, port, (password) ? password : "",
                         (nick1) ? nick1 : "", (nick2) ? nick2 : "", (nick3) ? nick3 : "",
                         (username) ? username : "", (realname) ? realname : "",
-                        (command) ? command : "", (autojoin) ? autojoin : "",
-                        (autorejoin) ? "on" : "off", (notify_levels) ? notify_levels : "",
+                        (hostname) ? hostname : "", (command) ? command : "",
+                        (autojoin) ? autojoin : "", (autorejoin) ? "on" : "off",
+                        (notify_levels) ? notify_levels : "",
                         (charset_decode_iso) ? charset_decode_iso : "",
                         (charset_decode_utf) ? charset_decode_utf : "",
                         (charset_encode) ? charset_encode : "");
@@ -406,6 +422,8 @@ server_new (char *name, int autoconnect, int autoreconnect, int autoreconnect_de
             (username) ? strdup (username) : strdup ("weechat");
         new_server->realname =
             (realname) ? strdup (realname) : strdup ("realname");
+        new_server->hostname =
+            (hostname) ? strdup (hostname) : NULL;
         new_server->command =
             (command) ? strdup (command) : NULL;
         new_server->command_delay = command_delay;
@@ -552,11 +570,14 @@ server_sendf (t_irc_server *server, char *fmt, ...)
     buffer[sizeof (buffer) - 1] = '\0';
     if ((size_buf < 0) || (size_buf > (int) (sizeof (buffer) - 1)))
         size_buf = strlen (buffer);
-#ifdef DEBUG
+    
     buffer[size_buf - 2] = '\0';
+    gui_printf_raw_data (server, 1, buffer);
+#ifdef DEBUG
     gui_printf (server->buffer, "[DEBUG] Sending to server >>> %s\n", buffer);
-    buffer[size_buf - 2] = '\r';
 #endif
+    buffer[size_buf - 2] = '\r';
+    
     if (server_send (server, buffer, strlen (buffer)) <= 0)
     {
         irc_display_prefix (server, server->buffer, PREFIX_ERROR);
@@ -726,6 +747,7 @@ server_msgq_flush ()
             
             if (ptr_data && ptr_data[0])
             {
+                gui_printf_raw_data (recv_msgq->server, 0, ptr_data);
 #ifdef DEBUG
                 gui_printf (NULL, "[DEBUG] data received from server: %s\n", ptr_data);
 #endif
@@ -960,51 +982,60 @@ server_child_read (t_irc_server *server)
             /* adress not found */
             case '1':
                 irc_display_prefix (server, server->buffer, PREFIX_ERROR);
-		if (cfg_proxy_use)
-		  gui_printf (server->buffer,
-			      _("%s proxy address \"%s\" not found\n"),
-			      WEECHAT_ERROR, server->address);
-		else
-		  gui_printf (server->buffer,
-			      _("%s address \"%s\" not found\n"),
-			      WEECHAT_ERROR, server->address);
+                if (cfg_proxy_use)
+                    gui_printf (server->buffer,
+                                _("%s proxy address \"%s\" not found\n"),
+                                WEECHAT_ERROR, server->address);
+                else
+                    gui_printf (server->buffer,
+                                _("%s address \"%s\" not found\n"),
+                                WEECHAT_ERROR, server->address);
                 server_close_connection (server);
                 server_reconnect_schedule (server);
                 break;
             /* IP address not found */
             case '2':
                 irc_display_prefix (server, server->buffer, PREFIX_ERROR);
-		if (cfg_proxy_use)
-		  gui_printf (server->buffer,
-			      _("%s proxy IP address not found\n"), WEECHAT_ERROR);
-		else
-		  gui_printf (server->buffer,
-			      _("%s IP address not found\n"), WEECHAT_ERROR);
+                if (cfg_proxy_use)
+                    gui_printf (server->buffer,
+                                _("%s proxy IP address not found\n"), WEECHAT_ERROR);
+                else
+                    gui_printf (server->buffer,
+                                _("%s IP address not found\n"), WEECHAT_ERROR);
                 server_close_connection (server);
                 server_reconnect_schedule (server);
                 break;
-	     /* connection refused */
+            /* connection refused */
             case '3':
                 irc_display_prefix (server, server->buffer, PREFIX_ERROR);
-		if (cfg_proxy_use)
-		  gui_printf (server->buffer,
-			      _("%s proxy connection refused\n"), WEECHAT_ERROR);
-		else
-		  gui_printf (server->buffer,
-			      _("%s connection refused\n"), WEECHAT_ERROR);
+                if (cfg_proxy_use)
+                    gui_printf (server->buffer,
+                                _("%s proxy connection refused\n"), WEECHAT_ERROR);
+                else
+                    gui_printf (server->buffer,
+                                _("%s connection refused\n"), WEECHAT_ERROR);
                 server_close_connection (server);
                 server_reconnect_schedule (server);
                 break;
-	    /* proxy fails to connect to server */
+            /* proxy fails to connect to server */
             case '4':
                 irc_display_prefix (server, server->buffer, PREFIX_ERROR);
-		gui_printf (server->buffer,
-			    _("%s proxy fails to establish connection to "
+                gui_printf (server->buffer,
+                            _("%s proxy fails to establish connection to "
                               "server (check username/password if used)\n"),
                             WEECHAT_ERROR);
                 server_close_connection (server);
                 server_reconnect_schedule (server);
                 break;
+            /* fails to set local hostname/IP */
+            case '5':
+                irc_display_prefix (server, server->buffer, PREFIX_ERROR);
+                gui_printf (server->buffer,
+                            _("%s unable to set local hostname/IP\n"),
+                            WEECHAT_ERROR);
+                server_close_connection (server);
+                server_reconnect_schedule (server);
+                break;
         }
     }
 }
@@ -1082,20 +1113,20 @@ pass_httpproxy(int sock, char *address, int port)
 {
 
     char buffer[256];
-    char authbuf[128]; // seems to be enougth to store username + password
-    char authbuf_base64[196]; // enougth to store base64 encoded authbuf
+    char authbuf[128];
+    char authbuf_base64[196];
     int n, m;
 
     if (strlen(cfg_proxy_username) > 0) 
     {
-        // authentification
+        /* authentification */
         snprintf(authbuf, sizeof(authbuf), "%s:%s", cfg_proxy_username, cfg_proxy_password);
         base64encode(authbuf, authbuf_base64);
         n = snprintf(buffer, sizeof(buffer), "CONNECT %s:%d HTTP/1.0\r\nProxy-Authorization: Basic %s\r\n\r\n", address, port, authbuf_base64);
     }
     else 
     {
-        // no authentification 
+        /* no authentification */
         n = snprintf(buffer, sizeof(buffer), "CONNECT %s:%d HTTP/1.0\r\n\r\n", address, port);
     }
   
@@ -1105,7 +1136,7 @@ pass_httpproxy(int sock, char *address, int port)
 
     n = recv(sock, buffer, sizeof(buffer), 0);
 
-    /* success result must be like : "HTTP/1.0 200 OK"  */
+    /* success result must be like: "HTTP/1.0 200 OK" */
     if (n < 12)
         return 1;
 
@@ -1166,7 +1197,7 @@ resolve (char *hostname, char *ip, int *version)
  */
 
 int
-pass_socks4proxy(int sock, char *address, int port, char *username)
+pass_socks4proxy (int sock, char *address, int port, char *username)
 {
     /* 
      * socks4 protocol is explain here: 
@@ -1248,28 +1279,27 @@ pass_socks5proxy(int sock, char *address, int port)
          *       - socks version (buffer[0]) = 5 => socks5
          *       - socks method  (buffer[1]) = 2 => authentication
          */
-
-        //if (!(buffer[0] == 5 && buffer[1] == 2))
+        
         if (buffer[0] != 5 || buffer[1] != 2)
             return 1;
-
+        
         /* authentication as in RFC 1929 */
         username_len = strlen(cfg_proxy_username);
         password_len = strlen(cfg_proxy_password);
-      
+        
         /* make username/password buffer */
         buffer[0] = 1;
         buffer[1] = (unsigned char) username_len;
         memcpy(buffer + 2, cfg_proxy_username, username_len);
         buffer[2 + username_len] = (unsigned char) password_len;
         memcpy(buffer + 3 + username_len, cfg_proxy_password, password_len);
-     
+        
         send (sock, buffer, 3 + username_len + password_len, 0);
-
+        
         /* server socks5 must respond with 2 bytes */
         if (recv (sock, buffer, 2, 0) != 2)
             return 1;
-
+        
         /* buffer[1] = auth state, must be 0 for success */
         if (buffer[1] != 0)
             return 1;
@@ -1354,13 +1384,13 @@ pass_socks5proxy(int sock, char *address, int port)
  */
 
 int
-pass_proxy(int sock, char *address, int port, char *username)
+pass_proxy (int sock, char *address, int port, char *username)
 {  
-    if (strcmp(cfg_proxy_type_values[cfg_proxy_type], "http") == 0)
+    if (strcmp (cfg_proxy_type_values[cfg_proxy_type], "http") == 0)
         return pass_httpproxy(sock, address, port);
-    if (strcmp(cfg_proxy_type_values[cfg_proxy_type], "socks4") == 0)
+    if (strcmp (cfg_proxy_type_values[cfg_proxy_type], "socks4") == 0)
         return pass_socks4proxy(sock, address, port, username);
-    if (strcmp(cfg_proxy_type_values[cfg_proxy_type], "socks5") == 0)
+    if (strcmp (cfg_proxy_type_values[cfg_proxy_type], "socks5") == 0)
         return pass_socks5proxy(sock, address, port);
 
     return 1;
@@ -1373,90 +1403,128 @@ pass_proxy(int sock, char *address, int port, char *username)
 int
 server_child (t_irc_server *server)
 {
-    struct addrinfo hints, *res;
+    struct addrinfo hints, *res, *res_local;
+    int rc;
     
     res = NULL;
+    res_local = NULL;
     
     if (cfg_proxy_use)
     {
-	memset (&hints, 0, sizeof (hints));
-	hints.ai_family = (cfg_proxy_ipv6) ? AF_INET6 : AF_INET;
-	hints.ai_socktype = SOCK_STREAM;
-	if (getaddrinfo (cfg_proxy_address, NULL, &hints, &res) !=0)
+        /* get info about server */
+        memset (&hints, 0, sizeof (hints));
+        hints.ai_family = (cfg_proxy_ipv6) ? AF_INET6 : AF_INET;
+        hints.ai_socktype = SOCK_STREAM;
+        if (getaddrinfo (cfg_proxy_address, NULL, &hints, &res) !=0)
         {
-	    write(server->child_write, "1", 1);
-	    return 0;
+            write(server->child_write, "1", 1);
+            return 0;
         }
         if (!res)
         {
             write(server->child_write, "1", 1);
-	    return 0;
+            return 0;
         }
-	if ((cfg_proxy_ipv6 && (res->ai_family != AF_INET6))
-	    || ((!cfg_proxy_ipv6 && (res->ai_family != AF_INET))))
+        if ((cfg_proxy_ipv6 && (res->ai_family != AF_INET6))
+            || ((!cfg_proxy_ipv6 && (res->ai_family != AF_INET))))
         {
-	    write (server->child_write, "2", 1);
+            write (server->child_write, "2", 1);
             freeaddrinfo (res);
-	    return 0;
+            return 0;
         }
         
-	if (cfg_proxy_ipv6)
+        if (cfg_proxy_ipv6)
             ((struct sockaddr_in6 *)(res->ai_addr))->sin6_port = htons (cfg_proxy_port);
-	else
+        else
             ((struct sockaddr_in *)(res->ai_addr))->sin_port = htons (cfg_proxy_port);
-	
-	if (connect (server->sock, res->ai_addr, res->ai_addrlen) != 0)
+
+        /* connect to server */
+        if (connect (server->sock, res->ai_addr, res->ai_addrlen) != 0)
         {
-	    write(server->child_write, "3", 1);
+            write(server->child_write, "3", 1);
             freeaddrinfo (res);
-	    return 0;
+            return 0;
         }
-	
-	if (pass_proxy(server->sock, server->address, server->port, server->username))
+        
+        if (pass_proxy(server->sock, server->address, server->port, server->username))
         {
-	    write(server->child_write, "4", 1);
+            write(server->child_write, "4", 1);
             freeaddrinfo (res);
-	    return 0;
+            return 0;
         }
     }
     else
     {
-	memset (&hints, 0, sizeof(hints));
-	hints.ai_family = (server->ipv6) ? AF_INET6 : AF_INET;
-	hints.ai_socktype = SOCK_STREAM;
-	if (getaddrinfo (server->address, NULL, &hints, &res) !=0)
+        /* set local hostname/IP if asked by user */
+        if (server->hostname && server->hostname[0])
         {
-	    write(server->child_write, "1", 1);
-	    return 0;
-        }
-        if (!res)
-        {
-            write(server->child_write, "1", 1);
-	    return 0;
-        }
-	if ((server->ipv6 && (res->ai_family != AF_INET6))
-	    || ((!server->ipv6 && (res->ai_family != AF_INET))))
-        {
-	    write(server->child_write, "2", 1);
-            freeaddrinfo (res);
-	    return 0;
+            memset (&hints, 0, sizeof(hints));
+            hints.ai_family = (server->ipv6) ? AF_INET6 : AF_INET;
+            hints.ai_socktype = SOCK_STREAM;
+            rc = getaddrinfo (server->hostname, NULL, &hints, &res_local);
+            if ((rc != 0) || !res_local
+                || (server->ipv6 && (res_local->ai_family != AF_INET6))
+                || ((!server->ipv6 && (res_local->ai_family != AF_INET))))
+            {
+                write (server->child_write, "5", 1);
+                if (res_local)
+                    freeaddrinfo (res_local);
+                return 0;
+            }
+            if (bind (server->sock, res_local->ai_addr, res_local->ai_addrlen) < 0)
+            {
+                write (server->child_write, "5", 1);
+                if (res_local)
+                    freeaddrinfo (res_local);
+                return 0;
+            }
         }
         
-	if (server->ipv6)
-            ((struct sockaddr_in6 *)(res->ai_addr))->sin6_port = htons (server->port);
-	else
-            ((struct sockaddr_in *)(res->ai_addr))->sin_port = htons (server->port);
-	
-	if (connect (server->sock, res->ai_addr, res->ai_addrlen) != 0)
+        /* get info about server */
+        memset (&hints, 0, sizeof(hints));
+        hints.ai_family = (server->ipv6) ? AF_INET6 : AF_INET;
+        hints.ai_socktype = SOCK_STREAM;
+        rc = getaddrinfo (server->address, NULL, &hints, &res);
+        if ((rc != 0) || !res)
         {
-	    write(server->child_write, "3", 1);
-            freeaddrinfo (res);
-	    return 0;
+            write (server->child_write, "1", 1);
+            if (res)
+                freeaddrinfo (res);
+            return 0;
+        }
+        if ((server->ipv6 && (res->ai_family != AF_INET6))
+            || ((!server->ipv6 && (res->ai_family != AF_INET))))
+        {
+            write (server->child_write, "2", 1);
+            if (res)
+                freeaddrinfo (res);
+            if (res_local)
+                freeaddrinfo (res_local);
+            return 0;
+        }
+        
+        /* connect to server */
+        if (server->ipv6)
+            ((struct sockaddr_in6 *)(res->ai_addr))->sin6_port = htons (server->port);
+        else
+            ((struct sockaddr_in *)(res->ai_addr))->sin_port = htons (server->port);
+        
+        if (connect (server->sock, res->ai_addr, res->ai_addrlen) != 0)
+        {
+            write (server->child_write, "3", 1);
+            if (res)
+                freeaddrinfo (res);
+            if (res_local)
+                freeaddrinfo (res_local);
+            return 0;
         }
     }
     
     write (server->child_write, "0", 1);
-    freeaddrinfo (res);
+    if (res)
+        freeaddrinfo (res);
+    if (res_local)
+        freeaddrinfo (res_local);
     return 0;
 }
 
@@ -1486,14 +1554,14 @@ server_connect (t_irc_server *server)
     irc_display_prefix (server, server->buffer, PREFIX_INFO);
     if (cfg_proxy_use)
       {
-	gui_printf (server->buffer,
-		    _("%s: connecting to server %s:%d%s%s via %s proxy %s:%d%s...\n"),
-		    PACKAGE_NAME, server->address, server->port,
-		    (server->ipv6) ? " (IPv6)" : "",
-		    (server->ssl) ? " (SSL)" : "",
-		    cfg_proxy_type_values[cfg_proxy_type], cfg_proxy_address, cfg_proxy_port,
-		    (cfg_proxy_ipv6) ? " (IPv6)" : "");
-	weechat_log_printf (_("Connecting to server %s:%d%s%s via %s proxy %s:%d%s...\n"),
+          gui_printf (server->buffer,
+                      _("%s: connecting to server %s:%d%s%s via %s proxy %s:%d%s...\n"),
+                      PACKAGE_NAME, server->address, server->port,
+                      (server->ipv6) ? " (IPv6)" : "",
+                      (server->ssl) ? " (SSL)" : "",
+                      cfg_proxy_type_values[cfg_proxy_type], cfg_proxy_address, cfg_proxy_port,
+                      (cfg_proxy_ipv6) ? " (IPv6)" : "");
+          weechat_log_printf (_("Connecting to server %s:%d%s%s via %s proxy %s:%d%s...\n"),
                             server->address, server->port,
                             (server->ipv6) ? " (IPv6)" : "",
                             (server->ssl) ? " (SSL)" : "",
@@ -1502,12 +1570,12 @@ server_connect (t_irc_server *server)
       }
     else
       {
-	gui_printf (server->buffer,
-		    _("%s: connecting to server %s:%d%s%s...\n"),
-		    PACKAGE_NAME, server->address, server->port,
-		    (server->ipv6) ? " (IPv6)" : "",
-		    (server->ssl) ? " (SSL)" : "");
-	weechat_log_printf (_("Connecting to server %s:%d%s%s...\n"),
+          gui_printf (server->buffer,
+                      _("%s: connecting to server %s:%d%s%s...\n"),
+                      PACKAGE_NAME, server->address, server->port,
+                      (server->ipv6) ? " (IPv6)" : "",
+                      (server->ssl) ? " (SSL)" : "");
+          weechat_log_printf (_("Connecting to server %s:%d%s%s...\n"),
                             server->address, server->port,
                             (server->ipv6) ? " (IPv6)" : "",
                             (server->ssl) ? " (SSL)" : "");
@@ -1633,8 +1701,9 @@ server_auto_connect (int auto_connect, int command_line)
         if ( ((command_line) && (ptr_server->command_line))
             || ((!command_line) && (auto_connect) && (ptr_server->autoconnect)) )
         {
-            (void) gui_buffer_new (gui_current_window, ptr_server, NULL, 0, 1);
-            gui_redraw_buffer (gui_current_window->buffer);
+            (void) gui_buffer_new (gui_current_window, ptr_server, NULL,
+                                   BUFFER_TYPE_STANDARD, 1);
+            gui_window_redraw_buffer (gui_current_window->buffer);
             if (!server_connect (ptr_server))
                 server_reconnect_schedule (ptr_server);
         }
@@ -1670,6 +1739,11 @@ server_disconnect (t_irc_server *server, int reconnect)
         gui_printf (server->buffer, _("Disconnected from server!\n"));
     }
     
+    if (server->nick_modes)
+    {
+        memset (server->nick_modes, ' ', NUM_NICK_MODES);
+        server->nick_modes[NUM_NICK_MODES] = '\0';
+    }
     server->is_away = 0;
     server->away_time = 0;
     server->lag = 0;
@@ -1682,7 +1756,7 @@ server_disconnect (t_irc_server *server, int reconnect)
     else
         server->reconnect_start = 0;
     
-    gui_redraw_buffer (gui_current_window->buffer);
+    gui_window_redraw_buffer (gui_current_window->buffer);
 }
 
 /*
@@ -1824,7 +1898,7 @@ server_check_away ()
             for (ptr_channel = ptr_server->channels; ptr_channel; ptr_channel = ptr_channel->next_channel)
             {
                 if (ptr_channel->type == CHANNEL_TYPE_CHANNEL)
-                    channel_check_away (ptr_server, ptr_channel);
+                    channel_check_away (ptr_server, ptr_channel, 0);
             }
         }
     }
@@ -1891,6 +1965,7 @@ server_print_log (t_irc_server *server)
     weechat_log_printf ("  ssl_connected . . . : %d\n",   server->ssl_connected);
     weechat_log_printf ("  unterminated_message: '%s'\n", server->unterminated_message);
     weechat_log_printf ("  nick. . . . . . . . : '%s'\n", server->nick);
+    weechat_log_printf ("  nick_modes. . . . . : '%s'\n", server->nick_modes);
     weechat_log_printf ("  reconnect_start . . : %ld\n",  server->reconnect_start);
     weechat_log_printf ("  reconnect_join. . . : %d\n",   server->reconnect_join);
     weechat_log_printf ("  is_away . . . . . . : %d\n",   server->is_away);
