@@ -28,6 +28,7 @@
 #include <time.h>
 #include <sys/time.h>
 #include <sys/types.h>
+#include <regex.h>
 
 #ifdef HAVE_GNUTLS
 #include <gnutls/gnutls.h>
@@ -45,7 +46,6 @@
 #define PREFIX_QUIT      "<--"
 #define PREFIX_ERROR     "=!="
 #define PREFIX_PLUGIN    "-P-"
-#define PREFIX_AWAY      "-A-"
 
 #define DEFAULT_IRC_PORT 6667
 
@@ -68,6 +68,7 @@ typedef struct t_irc_nick t_irc_nick;
 struct t_irc_nick
 {
     char *nick;                     /* nickname                               */
+    char *host;                     /* full hostname                          */
     int flags;                      /* chanowner/chanadmin (unrealircd),      */
                                     /* op, halfop, voice, away                */
     int color;                      /* color for nickname in chat window      */
@@ -79,11 +80,12 @@ struct t_irc_nick
 
 #define CHANNEL_PREFIX "#&+!"
 
-#define CHANNEL_TYPE_UNKNOWN -1
-#define CHANNEL_TYPE_CHANNEL 0
-#define CHANNEL_TYPE_PRIVATE 1
+#define CHANNEL_TYPE_UNKNOWN  -1
+#define CHANNEL_TYPE_CHANNEL  0
+#define CHANNEL_TYPE_PRIVATE  1
+#define CHANNEL_TYPE_DCC_CHAT 2
 
-#define NUM_CHANNEL_MODES       7
+#define NUM_CHANNEL_MODES       8
 #define CHANNEL_MODE_INVITE     0
 #define CHANNEL_MODE_KEY        1
 #define CHANNEL_MODE_LIMIT      2
@@ -91,6 +93,7 @@ struct t_irc_nick
 #define CHANNEL_MODE_NO_MSG_OUT 4
 #define CHANNEL_MODE_SECRET     5
 #define CHANNEL_MODE_TOPIC      6
+#define CHANNEL_MODE_PRIVATE    7
 #define CHANNEL_SET_MODE(channel, set, mode) \
     if (set) \
         channel->modes[mode] = channel_modes[mode]; \
@@ -110,6 +113,9 @@ struct t_irc_channel
     char *key;                      /* channel key (NULL if no key is set)    */
     int nicks_count;                /* # nicks on channel (0 if dcc/pv)       */
     int checking_away;              /* = 1 if checking away with WHO cmd      */
+    char *away_message;             /* to display away only once in private   */
+    int cycle;                      /* currently cycling (/part then /join)   */
+    int close;                      /* close request (/buffer close)          */
     t_irc_nick *nicks;              /* nicks on the channel                   */
     t_irc_nick *last_nick;          /* last nick on the channel               */
     t_gui_buffer *buffer;           /* GUI buffer allocated for channel       */
@@ -119,6 +125,20 @@ struct t_irc_channel
 
 /* server types */
 
+#define NUM_NICK_MODES           7
+#define NICK_MODE_AWAY           0
+#define NICK_MODE_INVISIBLE      1
+#define NICK_MODE_WALLOPS        2
+#define NICK_MODE_RESTRICTED     3
+#define NICK_MODE_OPERATOR       4
+#define NICK_MODE_LOCAL_OPERATOR 5
+#define NICK_MODE_SERVER_NOTICES 6
+#define NICK_SET_MODE(server, set, mode) \
+    if (set) \
+        server->nick_modes[mode] = nick_modes[mode]; \
+    else \
+        server->nick_modes[mode] = ' ';
+
 typedef struct t_irc_server t_irc_server;
 
 struct t_irc_server
@@ -139,6 +159,7 @@ struct t_irc_server
     char *nick3;                    /* 2nd alternate nickname                 */
     char *username;                 /* user name                              */
     char *realname;                 /* real name                              */
+    char *hostname;                 /* custom hostname                        */
     char *command;                  /* command to run once connected          */
     int command_delay;              /* delay after execution of command       */
     char *autojoin;                 /* channels to automatically join         */
@@ -160,6 +181,7 @@ struct t_irc_server
 #endif
     char *unterminated_message;     /* beginning of a message in input buf    */
     char *nick;                     /* current nickname                       */
+    char *nick_modes;               /* nick modes                             */
     time_t reconnect_start;         /* this time + delay = reconnect time     */
     int reconnect_join;             /* 1 if channels opened to rejoin         */
     int is_away;                    /* 1 is user is marker as away            */
@@ -167,6 +189,7 @@ struct t_irc_server
     int lag;                        /* lag (in milliseconds)                  */
     struct timeval lag_check_time;  /* last time lag was checked (ping sent)  */
     time_t lag_next_check;          /* time for next check                    */
+    regex_t *cmd_list_regexp;       /* compiled Regular Expression for /list  */ 
     t_gui_buffer *buffer;           /* GUI buffer allocated for server        */
     t_gui_buffer *saved_buffer;     /* channel before jumping to next server  */
     t_irc_channel *channels;        /* opened channels on server              */
@@ -188,10 +211,12 @@ struct t_irc_command
     char *completion_template;      /* template for completion                */
                                     /* NULL=no completion, ""=default (nick)  */
     int min_arg, max_arg;           /* min & max number of arguments          */
-    int need_connection;            /* = 1 if cmd needs server connection     */
-    int (*cmd_function_args)(t_irc_server *, int, char **);
+    int charset_conversion;         /* = 1 if cmd args are converted before   */
+                                    /* they're sent to server                 */
+    int needs_connection;           /* = 1 if cmd needs server connection     */
+    int (*cmd_function_args)(t_irc_server *, t_irc_channel *, int, char **);
                                     /* function called when user enters cmd   */
-    int (*cmd_function_1arg)(t_irc_server *, char *);
+    int (*cmd_function_1arg)(t_irc_server *, t_irc_channel *, char *);
                                     /* function called when user enters cmd   */
     int (*recv_function)(t_irc_server *, char *, char *, char *);
                                     /* function called when cmd is received   */
@@ -293,6 +318,7 @@ extern int check_away;
 extern t_irc_dcc *dcc_list;
 extern char *dcc_status_string[6];
 extern char *channel_modes;
+extern char *nick_modes;
 extern char *ignore_types[];
 extern t_irc_ignore *irc_ignore;
 extern t_irc_ignore *irc_last_ignore;
@@ -307,8 +333,8 @@ extern void server_free (t_irc_server *);
 extern void server_free_all ();
 extern t_irc_server *server_new (char *, int, int, int, int, char *, int, int, int,
                                  char *, char *, char *, char *, char *, char *,
-                                 char *, int, char *, int, char *, char *, char *,
-                                 char *);
+                                 char *, char *, int, char *, int, char *, char *,
+                                 char *, char *);
 extern char *server_get_charset_decode_iso (t_irc_server *);
 extern char *server_get_charset_decode_utf (t_irc_server *);
 extern char *server_get_charset_encode (t_irc_server *);
@@ -345,6 +371,9 @@ extern t_irc_channel *channel_new (t_irc_server *, int, char *);
 extern void channel_free (t_irc_server *, t_irc_channel *);
 extern void channel_free_all (t_irc_server *);
 extern t_irc_channel *channel_search (t_irc_server *, char *);
+extern t_irc_channel *channel_search_any (t_irc_server *, char *);
+extern t_irc_channel *channel_search_any_without_buffer (t_irc_server *, char *);
+extern t_irc_channel *channel_search_dcc (t_irc_server *, char *);
 extern int string_is_channel (char *);
 extern char *channel_get_charset_decode_iso (t_irc_server *, t_irc_channel *);
 extern char *channel_get_charset_decode_utf (t_irc_server *, t_irc_channel *);
@@ -352,10 +381,9 @@ extern char *channel_get_charset_encode (t_irc_server *, t_irc_channel *);
 extern char *channel_iconv_decode (t_irc_server *, t_irc_channel *, char *);
 extern char *channel_iconv_encode (t_irc_server *, t_irc_channel *, char *);
 extern void channel_remove_away (t_irc_channel *);
-extern void channel_check_away (t_irc_server *, t_irc_channel *);
+extern void channel_check_away (t_irc_server *, t_irc_channel *, int);
 extern void channel_set_away (t_irc_channel *, char *, int);
 extern int channel_create_dcc (t_irc_dcc *);
-extern void channel_remove_dcc (t_irc_dcc *);
 extern int channel_get_notify_level (t_irc_server *, t_irc_channel *);
 extern void channel_set_notify_level (t_irc_server *, t_irc_channel *, int);
 extern void channel_print_log (t_irc_channel *);
@@ -380,6 +408,7 @@ extern void nick_print_log (t_irc_nick *);
 extern void dcc_redraw (int);
 extern void dcc_free (t_irc_dcc *);
 extern void dcc_close (t_irc_dcc *, int);
+extern void dcc_chat_remove_channel (t_irc_channel *);
 extern void dcc_accept (t_irc_dcc *);
 extern void dcc_accept_resume (t_irc_server *, char *, int, unsigned long);
 extern void dcc_start_resume (t_irc_server *, char *, int, unsigned long);
@@ -394,76 +423,76 @@ extern void dcc_print_log (t_irc_dcc *);
 
 /* IRC display (irc-diplay.c) */
 
+extern void irc_find_context (t_irc_server *, t_irc_channel *,
+                              t_gui_window **, t_gui_buffer **);
 extern void irc_display_prefix (t_irc_server *, t_gui_buffer *, char *);
 extern void irc_display_nick (t_gui_buffer *, t_irc_nick *, char *, int,
                               int, int, int);
 extern void irc_display_away (t_irc_server *, char *, char *);
-extern void irc_display_mode (t_irc_server *, t_gui_buffer *, char *, char,
-                              char *, char *, char *, char *);
+extern void irc_display_mode (t_irc_server *, t_gui_buffer *, char *, char *,
+                              char, char *, char *, char *, char *);
 extern void irc_display_server (t_irc_server *ptr_server);
 
 /* IRC commands issued by user (irc-send.c) */
 
 extern void irc_login (t_irc_server *);
-extern int irc_cmd_send_admin (t_irc_server *, char *);
-extern int irc_cmd_send_ame (t_irc_server *, char *);
-extern int irc_cmd_send_amsg (t_irc_server *, char *);
-extern int irc_cmd_send_away (t_irc_server *, char *);
-extern int irc_cmd_send_ban (t_irc_server *, char *);
-extern int irc_cmd_send_ctcp (t_irc_server *, char *);
-extern int irc_cmd_send_dcc (t_irc_server *, char *);
-extern int irc_cmd_send_dehalfop (t_irc_server *, int, char **);
-extern int irc_cmd_send_deop (t_irc_server *, int, char **);
-extern int irc_cmd_send_devoice (t_irc_server *, int, char **);
-extern int irc_cmd_send_die (t_irc_server *, char *);
-extern int irc_cmd_send_halfop (t_irc_server *, int, char **);
-extern int irc_cmd_send_info (t_irc_server *, char *);
-extern int irc_cmd_send_invite (t_irc_server *, int, char **);
-extern int irc_cmd_send_ison (t_irc_server *, char *);
-extern int irc_cmd_send_join (t_irc_server *, char *);
-extern int irc_cmd_send_kick (t_irc_server *, char *);
-extern int irc_cmd_send_kickban (t_irc_server *, char *);
-extern int irc_cmd_send_kill (t_irc_server *, char *);
-extern int irc_cmd_send_links (t_irc_server *, char *);
-extern int irc_cmd_send_list (t_irc_server *, char *);
-extern int irc_cmd_send_lusers (t_irc_server *, char *);
-extern int irc_send_me (t_irc_server *, t_irc_channel *, char *);
-extern int irc_send_me_all_channels (t_irc_server *, char *);
-extern int irc_cmd_send_me (t_irc_server *, char *);
-extern int irc_cmd_send_mode (t_irc_server *, char *);
-extern int irc_cmd_send_motd (t_irc_server *, char *);
-extern int irc_cmd_send_msg (t_irc_server *, char *);
-extern int irc_cmd_send_names (t_irc_server *, char *);
-extern int irc_cmd_send_nick (t_irc_server *, int, char **);
-extern int irc_cmd_send_notice (t_irc_server *, char *);
-extern int irc_cmd_send_op (t_irc_server *, int, char **);
-extern int irc_cmd_send_oper (t_irc_server *, char *);
-extern int irc_cmd_send_part (t_irc_server *, char *);
-extern int irc_cmd_send_ping (t_irc_server *, char *);
-extern int irc_cmd_send_pong (t_irc_server *, char *);
-extern int irc_cmd_send_query (t_irc_server *, char *);
-extern int irc_cmd_send_quit (t_irc_server *, char *);
-extern int irc_cmd_send_quote (t_irc_server *, char *);
-extern int irc_cmd_send_rehash (t_irc_server *, char *);
-extern int irc_cmd_send_restart (t_irc_server *, char *);
-extern int irc_cmd_send_service (t_irc_server *, char *);
-extern int irc_cmd_send_servlist (t_irc_server *, char *);
-extern int irc_cmd_send_squery (t_irc_server *, char *);
-extern int irc_cmd_send_squit (t_irc_server *, char *);
-extern int irc_cmd_send_stats (t_irc_server *, char *);
-extern int irc_cmd_send_summon (t_irc_server *, char *);
-extern int irc_cmd_send_time (t_irc_server *, char *);
-extern int irc_cmd_send_topic (t_irc_server *, char *);
-extern int irc_cmd_send_trace (t_irc_server *, char *);
-extern int irc_cmd_send_unban (t_irc_server *, char *);
-extern int irc_cmd_send_userhost (t_irc_server *, char *);
-extern int irc_cmd_send_users (t_irc_server *, char *);
-extern int irc_cmd_send_version (t_irc_server *, char *);
-extern int irc_cmd_send_voice (t_irc_server *, int, char **);
-extern int irc_cmd_send_wallops (t_irc_server *, char *);
-extern int irc_cmd_send_who (t_irc_server *, char *);
-extern int irc_cmd_send_whois (t_irc_server *, char *);
-extern int irc_cmd_send_whowas (t_irc_server *, char *);
+extern int irc_cmd_send_admin (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_ame (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_amsg (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_away (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_ban (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_ctcp (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_cycle (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_dehalfop (t_irc_server *, t_irc_channel *, int, char **);
+extern int irc_cmd_send_deop (t_irc_server *, t_irc_channel *, int, char **);
+extern int irc_cmd_send_devoice (t_irc_server *, t_irc_channel *, int, char **);
+extern int irc_cmd_send_die (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_halfop (t_irc_server *, t_irc_channel *, int, char **);
+extern int irc_cmd_send_info (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_invite (t_irc_server *, t_irc_channel *, int, char **);
+extern int irc_cmd_send_ison (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_join (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_kick (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_kickban (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_kill (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_links (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_list (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_lusers (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_me (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_mode (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_motd (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_msg (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_names (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_nick (t_irc_server *, t_irc_channel *, int, char **);
+extern int irc_cmd_send_notice (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_op (t_irc_server *, t_irc_channel *, int, char **);
+extern int irc_cmd_send_oper (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_part (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_ping (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_pong (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_query (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_quit (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_quote (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_rehash (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_restart (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_service (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_servlist (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_squery (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_squit (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_stats (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_summon (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_time (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_topic (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_trace (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_unban (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_userhost (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_users (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_version (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_voice (t_irc_server *, t_irc_channel *, int, char **);
+extern int irc_cmd_send_wallops (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_who (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_whois (t_irc_server *, t_irc_channel *, char *);
+extern int irc_cmd_send_whowas (t_irc_server *, t_irc_channel *, char *);
 
 /* IRC commands executed when received from server (irc-recv.c) */
 
@@ -482,6 +511,7 @@ extern int irc_cmd_recv_ping (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_pong (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_privmsg (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_quit (t_irc_server *, char *, char *, char *);
+extern int irc_cmd_recv_server_mode_reason (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_server_msg (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_server_reply (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_topic (t_irc_server *, char *, char *, char *);
@@ -523,6 +553,7 @@ extern int irc_cmd_recv_365 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_366 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_367 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_368 (t_irc_server *, char *, char *, char *);
+extern int irc_cmd_recv_378 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_433 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_438 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_671 (t_irc_server *, char *, char *, char *);
