@@ -51,7 +51,7 @@ t_weechat_plugin *last_weechat_plugin = NULL;
  * plugin_find_server_channel: find server/channel for command execution
  */
 
-void
+int
 plugin_find_server_channel (char *server, char *channel,
                             t_irc_server **ptr_server,
                             t_irc_channel **ptr_channel)
@@ -70,7 +70,11 @@ plugin_find_server_channel (char *server, char *channel,
     else
     {
         if (server && server[0])
+        {
             (*ptr_server) = server_search (server);
+            if (!(*ptr_server))
+                return -1;
+        }
         else
         {
             (*ptr_server) = SERVER(gui_current_window->buffer);
@@ -82,8 +86,11 @@ plugin_find_server_channel (char *server, char *channel,
         {
             if ((*ptr_server))
                 (*ptr_channel) = channel_search ((*ptr_server), channel);
+            if (!(*ptr_channel))
+                return -1;
         }
     }
+    return 0;
 }
 
 /*
@@ -192,6 +199,9 @@ plugin_msg_handler_add (t_weechat_plugin *plugin, char *irc_command,
         new_handler->description = NULL;
         new_handler->arguments = NULL;
         new_handler->arguments_description = NULL;
+        new_handler->completion_template = NULL;
+        new_handler->interval = 0;
+        new_handler->remaining = 0;
         new_handler->handler = handler_func;
         new_handler->handler_args = (handler_args) ? strdup (handler_args) : NULL;
         new_handler->handler_pointer = handler_pointer;
@@ -252,6 +262,16 @@ plugin_cmd_handler_add (t_weechat_plugin *plugin, char *command,
                     WEECHAT_ERROR, plugin->name, command);
         return NULL;
     }
+    
+    if (ascii_strcasecmp (command, "builtin") == 0)
+    {
+        irc_display_prefix (NULL, NULL, PREFIX_ERROR);
+        gui_printf (NULL,
+                    _("%s plugin %s: unable to add handler for \"%s\" command "
+                      "(forbidden)\n"),
+                    WEECHAT_ERROR, plugin->name, command);
+        return NULL;
+    }
         
     new_handler = (t_plugin_handler *)malloc (sizeof (t_plugin_handler));
     if (new_handler)
@@ -263,6 +283,8 @@ plugin_cmd_handler_add (t_weechat_plugin *plugin, char *command,
         new_handler->arguments = (arguments) ? strdup (arguments) : NULL;
         new_handler->arguments_description = (arguments_description) ? strdup (arguments_description) : NULL;
         new_handler->completion_template = (completion_template) ? strdup (completion_template) : NULL;
+        new_handler->interval = 0;
+        new_handler->remaining = 0;
         new_handler->handler = handler_func;
         new_handler->handler_args = (handler_args) ? strdup (handler_args) : NULL;
         new_handler->handler_pointer = handler_pointer;
@@ -292,6 +314,118 @@ plugin_cmd_handler_add (t_weechat_plugin *plugin, char *command,
     return new_handler;
 }
 
+/*
+ * plugin_timer_handler_add: add a timer handler
+ *                           arguments:
+ *                             1. the plugin pointer
+ *                             2. the interval between two calls
+ *                             3. the handler function
+ *                             4. handler args: a string given to
+ *                                handler when called (used by scripts)
+ *                             5. handler pointer: a pointer given to
+ *                                handler when called (used by scripts)
+ */
+
+t_plugin_handler *
+plugin_timer_handler_add (t_weechat_plugin *plugin, int interval,
+                          t_plugin_handler_func *handler_func,
+                          char *handler_args, void *handler_pointer)
+{
+    t_plugin_handler *new_handler;
+    
+    new_handler = (t_plugin_handler *)malloc (sizeof (t_plugin_handler));
+    if (new_handler)
+    {
+        new_handler->type = HANDLER_TIMER;
+        new_handler->irc_command = NULL;
+        new_handler->command = NULL;
+        new_handler->description = NULL;
+        new_handler->arguments = NULL;
+        new_handler->arguments_description = NULL;
+        new_handler->completion_template = NULL;
+        new_handler->interval = interval;
+        new_handler->remaining = interval;
+        new_handler->handler = handler_func;
+        new_handler->handler_args = (handler_args) ? strdup (handler_args) : NULL;
+        new_handler->handler_pointer = handler_pointer;
+        new_handler->running = 0;
+        
+        /* add new handler to list */
+        new_handler->prev_handler = plugin->last_handler;
+        new_handler->next_handler = NULL;
+        if (plugin->handlers)
+            (plugin->last_handler)->next_handler = new_handler;
+        else
+            plugin->handlers = new_handler;
+        plugin->last_handler = new_handler;
+    }
+    else
+    {
+        irc_display_prefix (NULL, NULL, PREFIX_ERROR);
+        gui_printf (NULL,
+                    _("%s plugin %s: unable to add timer handler (not enough memory)\n"),
+                    WEECHAT_ERROR, plugin->name);
+        return NULL;
+    }
+    return new_handler;
+}
+
+/*
+ * plugin_keyboard_handler_add: add a timer handler
+ *                              arguments:
+ *                                1. the plugin pointer
+ *                                2. the interval between two calls
+ *                                3. the handler function
+ *                                4. handler args: a string given to
+ *                                   handler when called (used by scripts)
+ *                                5. handler pointer: a pointer given to
+ *                                   handler when called (used by scripts)
+ */
+
+t_plugin_handler *
+plugin_keyboard_handler_add (t_weechat_plugin *plugin,
+                             t_plugin_handler_func *handler_func,
+                             char *handler_args, void *handler_pointer)
+{
+    t_plugin_handler *new_handler;
+    
+    new_handler = (t_plugin_handler *)malloc (sizeof (t_plugin_handler));
+    if (new_handler)
+    {
+        new_handler->type = HANDLER_KEYBOARD;
+        new_handler->irc_command = NULL;
+        new_handler->command = NULL;
+        new_handler->description = NULL;
+        new_handler->arguments = NULL;
+        new_handler->arguments_description = NULL;
+        new_handler->completion_template = NULL;
+        new_handler->interval = 0;
+        new_handler->remaining = 0;
+        new_handler->handler = handler_func;
+        new_handler->handler_args = (handler_args) ? strdup (handler_args) : NULL;
+        new_handler->handler_pointer = handler_pointer;
+        new_handler->running = 0;
+        
+        /* add new handler to list */
+        new_handler->prev_handler = plugin->last_handler;
+        new_handler->next_handler = NULL;
+        if (plugin->handlers)
+            (plugin->last_handler)->next_handler = new_handler;
+        else
+            plugin->handlers = new_handler;
+        plugin->last_handler = new_handler;
+    }
+    else
+    {
+        irc_display_prefix (NULL, NULL, PREFIX_ERROR);
+        gui_printf (NULL,
+                    _("%s plugin %s: unable to add keyboard handler (not enough memory)\n"),
+                    WEECHAT_ERROR, plugin->name);
+        return NULL;
+    }
+    return new_handler;
+}
+
 /*
  * plugin_msg_handler_exec: execute a message handler
  *                          return: code for informing WeeChat whether message
@@ -304,6 +438,11 @@ plugin_msg_handler_exec (char *server, char *irc_command, char *irc_message)
     t_weechat_plugin *ptr_plugin;
     t_plugin_handler *ptr_handler;
     int return_code, final_return_code;
+    char *argv[3] = { NULL, NULL, NULL };
+    
+    argv[0] = server;
+    argv[1] = irc_command;
+    argv[2] = irc_message;
     
     final_return_code = PLUGIN_RC_OK;
     
@@ -320,9 +459,7 @@ plugin_msg_handler_exec (char *server, char *irc_command, char *irc_message)
                 {
                     ptr_handler->running = 1;
                     return_code = ((int) (ptr_handler->handler) (ptr_plugin,
-                                                                 server,
-                                                                 irc_command,
-                                                                 irc_message,
+                                                                 3, argv,
                                                                  ptr_handler->handler_args,
                                                                  ptr_handler->handler_pointer));
                     ptr_handler->running = 0;
@@ -344,7 +481,8 @@ plugin_msg_handler_exec (char *server, char *irc_command, char *irc_message)
 
 /*
  * plugin_cmd_handler_exec: execute a command handler
- *                          return: 1 if handler executed, 0 if no handler found
+ *                          return: 1 if handler executed, 0 if failed,
+ *                                  -1 if no handler found
  */
 
 int
@@ -353,6 +491,12 @@ plugin_cmd_handler_exec (char *server, char *command, char *arguments)
     t_weechat_plugin *ptr_plugin;
     t_plugin_handler *ptr_handler;
     int return_code;
+    char empty_arg[1] = { '\0' };
+    char *argv[3] = { NULL, NULL, NULL };
+    
+    argv[0] = server;
+    argv[1] = command;
+    argv[2] = (arguments) ? arguments : empty_arg;
     
     for (ptr_plugin = weechat_plugins; ptr_plugin;
          ptr_plugin = ptr_plugin->next_plugin)
@@ -367,9 +511,7 @@ plugin_cmd_handler_exec (char *server, char *command, char *arguments)
                 {
                     ptr_handler->running = 1;
                     return_code = (int) (ptr_handler->handler) (ptr_plugin,
-                                                                server,
-                                                                command,
-                                                                arguments,
+                                                                3, argv,
                                                                 ptr_handler->handler_args,
                                                                 ptr_handler->handler_pointer);
                     ptr_handler->running = 0;
@@ -379,7 +521,89 @@ plugin_cmd_handler_exec (char *server, char *command, char *arguments)
         }
     }
     
-    return 0;
+    return -1;
+}
+
+/*
+ * plugin_timer_handler_exec: check timer handlers and execute functions if needed
+ *                            return: PLUGIN_RC_OK if all ok
+ *                                    PLUGIN_RC_KO if at least one handler failed
+ */
+
+int
+plugin_timer_handler_exec ()
+{
+    t_weechat_plugin *ptr_plugin;
+    t_plugin_handler *ptr_handler;
+    int return_code, final_return_code;
+    
+    final_return_code = PLUGIN_RC_OK;
+    
+    for (ptr_plugin = weechat_plugins; ptr_plugin;
+         ptr_plugin = ptr_plugin->next_plugin)
+    {
+        for (ptr_handler = ptr_plugin->handlers;
+             ptr_handler; ptr_handler = ptr_handler->next_handler)
+        {
+            if (ptr_handler->type == HANDLER_TIMER)
+            {
+                ptr_handler->remaining--;
+                if (ptr_handler->remaining <= 0)
+                {
+                    return_code = ((int) (ptr_handler->handler) (ptr_plugin,
+                                                                 0, NULL,
+                                                                 ptr_handler->handler_args,
+                                                                 ptr_handler->handler_pointer));
+                    ptr_handler->remaining = ptr_handler->interval;
+                    if (return_code == PLUGIN_RC_KO)
+                        final_return_code = PLUGIN_RC_KO;
+                }
+            }
+        }
+    }
+    
+    return final_return_code;
+}
+
+/*
+ * plugin_keyboard_handler_exec: execute all keyboard handlers
+ *                               return: PLUGIN_RC_OK if all ok
+ *                                       PLUGIN_RC_KO if at least one handler failed
+ */
+
+int
+plugin_keyboard_handler_exec (char *key, char *input_before, char *input_after)
+{
+    t_weechat_plugin *ptr_plugin;
+    t_plugin_handler *ptr_handler;
+    int return_code, final_return_code;
+    char *argv[3] = { NULL, NULL, NULL };
+    
+    argv[0] = key;
+    argv[1] = input_before;
+    argv[2] = input_after;
+    
+    final_return_code = PLUGIN_RC_OK;
+    
+    for (ptr_plugin = weechat_plugins; ptr_plugin;
+         ptr_plugin = ptr_plugin->next_plugin)
+    {
+        for (ptr_handler = ptr_plugin->handlers;
+             ptr_handler; ptr_handler = ptr_handler->next_handler)
+        {
+            if (ptr_handler->type == HANDLER_KEYBOARD)
+            {
+                return_code = ((int) (ptr_handler->handler) (ptr_plugin,
+                                                             3, argv,
+                                                             ptr_handler->handler_args,
+                                                             ptr_handler->handler_pointer));
+                if (return_code == PLUGIN_RC_KO)
+                    final_return_code = PLUGIN_RC_KO;
+            }
+        }
+    }
+    
+    return final_return_code;
 }
 
 /*
@@ -613,11 +837,15 @@ plugin_load (char *filename)
         new_plugin->exec_on_files = &weechat_plugin_exec_on_files;
         new_plugin->msg_handler_add = &weechat_plugin_msg_handler_add;
         new_plugin->cmd_handler_add = &weechat_plugin_cmd_handler_add;
+        new_plugin->timer_handler_add = &weechat_plugin_timer_handler_add;
+        new_plugin->keyboard_handler_add = &weechat_plugin_keyboard_handler_add;
         new_plugin->handler_remove = &weechat_plugin_handler_remove;
         new_plugin->handler_remove_all = &weechat_plugin_handler_remove_all;
-        new_plugin->printf = &weechat_plugin_printf;
-        new_plugin->printf_server = &weechat_plugin_printf_server;
-        new_plugin->infobar_printf = &weechat_plugin_infobar_printf;
+        new_plugin->print = &weechat_plugin_print;
+        new_plugin->print_server = &weechat_plugin_print_server;
+        new_plugin->print_infobar = &weechat_plugin_print_infobar;
+        new_plugin->infobar_remove = &weechat_plugin_infobar_remove;
+	new_plugin->log = &weechat_plugin_log;
         new_plugin->exec_command = &weechat_plugin_exec_command;
         new_plugin->get_info = &weechat_plugin_get_info;
         new_plugin->get_dcc_info = &weechat_plugin_get_dcc_info;
@@ -626,6 +854,13 @@ plugin_load (char *filename)
         new_plugin->set_config = &weechat_plugin_set_config;
         new_plugin->get_plugin_config = &weechat_plugin_get_plugin_config;
         new_plugin->set_plugin_config = &weechat_plugin_set_plugin_config;
+	new_plugin->get_server_info = &weechat_plugin_get_server_info;
+	new_plugin->free_server_info = &weechat_plugin_free_server_info;
+	new_plugin->get_channel_info = &weechat_plugin_get_channel_info;
+	new_plugin->free_channel_info = &weechat_plugin_free_channel_info;
+	new_plugin->get_nick_info = &weechat_plugin_get_nick_info;
+	new_plugin->free_nick_info = &weechat_plugin_free_nick_info;
+        new_plugin->input_color = &weechat_plugin_input_color;
         
         /* handlers */
         new_plugin->handlers = NULL;
@@ -709,7 +944,8 @@ int plugin_auto_load_file (t_weechat_plugin *plugin, char *filename)
 
 void plugin_auto_load ()
 {
-    char *ptr_home, *dir_name, *list_plugins, *pos, *pos2;
+    char *ptr_home, *dir_name, *plugins_path, *plugins_path2;
+    char *list_plugins, *pos, *pos2;
     
     if (cfg_plugins_autoload && cfg_plugins_autoload[0])
     {
@@ -718,20 +954,20 @@ void plugin_auto_load ()
             /* auto-load plugins in WeeChat home dir */
             if (cfg_plugins_path && cfg_plugins_path[0])
             {
-                if (cfg_plugins_path[0] == '~')
-                {
-                    ptr_home = getenv ("HOME");
-                    dir_name = (char *)malloc (strlen (cfg_plugins_path) + strlen (ptr_home) + 2);
-                    if (dir_name)
-                    {
-                        strcpy (dir_name, ptr_home);
-                        strcat (dir_name, cfg_plugins_path + 1);
-                        plugin_exec_on_files (NULL, dir_name, &plugin_auto_load_file);
-                        free (dir_name);
-                    }
-                }
-                else
-                    plugin_exec_on_files (NULL, cfg_plugins_path, &plugin_auto_load_file);
+                ptr_home = getenv ("HOME");
+                plugins_path = weechat_strreplace (cfg_plugins_path, "~", ptr_home);
+                plugins_path2 = weechat_strreplace ((plugins_path) ?
+                                                    plugins_path : cfg_plugins_path,
+                                                    "%h", weechat_home);
+                plugin_exec_on_files (NULL,
+                                      (plugins_path2) ?
+                                      plugins_path2 : ((plugins_path) ?
+                                                       plugins_path : cfg_plugins_path),
+                                      &plugin_auto_load_file);
+                if (plugins_path)
+                    free (plugins_path);
+                if (plugins_path2)
+                    free (plugins_path2);
             }
     
             /* auto-load plugins in WeeChat global lib dir */
