@@ -22,6 +22,7 @@
 #define __WEECHAT_PLUGINS_H 1
 
 #include "weechat-plugin.h"
+#include "plugins-config.h"
 #include "../irc/irc.h"
 #include "../gui/gui.h"
 
@@ -31,8 +32,8 @@ typedef void (t_weechat_end_func) (t_weechat_plugin *);
 extern t_weechat_plugin *weechat_plugins;
 extern t_weechat_plugin *last_weechat_plugin;
 
-extern void plugin_find_server_channel (char *, char *,
-                                        t_irc_server **, t_irc_channel **);
+extern int plugin_find_server_channel (char *, char *,
+                                       t_irc_server **, t_irc_channel **);
 extern void plugin_exec_on_files (t_weechat_plugin *, char *,
                                   int (*)(t_weechat_plugin *, char *));
 extern t_weechat_plugin *plugin_search (char *);
@@ -44,8 +45,16 @@ extern t_plugin_handler *plugin_cmd_handler_add (t_weechat_plugin *, char *,
                                                  char *,
                                                  t_plugin_handler_func *,
                                                  char *, void *);
+extern t_plugin_handler *plugin_timer_handler_add (t_weechat_plugin *, int,
+                                                   t_plugin_handler_func *,
+                                                   char *, void *);
+extern t_plugin_handler *plugin_keyboard_handler_add (t_weechat_plugin *,
+                                                      t_plugin_handler_func *,
+                                                      char *, void *);
 extern int plugin_msg_handler_exec (char *, char *, char *);
 extern int plugin_cmd_handler_exec (char *, char *, char *);
+extern int plugin_timer_handler_exec ();
+extern int plugin_keyboard_handler_exec (char *, char *, char *);
 extern void plugin_handler_remove (t_weechat_plugin *,
                                    t_plugin_handler *);
 extern void plugin_handler_remove_all (t_weechat_plugin *);
